/* just test cscmm.c */

#include <stdio.h>
#include "spblas.h"
#include "spblas_i.h"
#include "cblas.h"

/* initialize a matrix */
void ArraySetToValue(double *A, int ldA, int outer , int inner, double value)
{
  double *p, *pend, *t, *tend;

  pend = A + outer*ldA;
  for (p=A; p<pend; p+=ldA)  {
    tend = p+inner;
    for (t=p; t<tend; *t++ = value);
  }
}

/* print a dense matrix */
void ArrayPrint(double *A, int ldA, int outer , int inner, FILE *output)
{
  double *p, *pend, *t, *tend;
  
  fprintf(output, "%d %d\n", outer, inner);

  pend = A + outer*ldA;

  for (p=A; p<pend; p+=ldA) {
    tend = p+inner;
    for (t=p; t<tend; t++) 
      fprintf(output,"%f ",*t);
    fprintf(output,"\n");
  }
}

int main(int argc, char *argv[])
{
	DCSC A = NULL;

	int M = 5;
	int N = 5;
	int nz = 5;

	double val[] = { 1.1, 2.1, 3.1, 4.1, 5.1 };
	int indx[]   = {  0,   1,   2,   3,   4 };
	int pntrb[]  = {  0,   1,   2,   3,   4 };
	int pntre[]  = {  1,   2,   3,   4,   5 };
	int prop = blas_zero_base;

	/*
	double val[] = { 1.1, 2.1, 3.1, 4.1, 5.1 };
	int indx[]   = {  1,   2,   3,   4,   5 };
	int pntrb[]  = {  1,   2,   3,   4,   5 };
	int pntre[]  = {  2,   3,   4,   5,   6 };
	int prop = blas_one_base;
	*/

	double *B;

	int ldb = 5;
	double alpha = 1.0;
	int k = 5;

	double beta = 0.0;
	double *C;
	int ldc = 5;
	
	int order = CblasRowMajor;
	int transpose = CblasNoTrans;

	B = (double *) malloc(M * k * sizeof(double));
	C = (double *) malloc(M * k * sizeof(double));

	ArraySetToValue(B, ldb, k , M, 1.0);
	ArraySetToValue(C, ldc, k , M, 0.0);

	A = create_DCSC(M, N, nz, val, indx, pntrb, pntre, prop); 
	DCSC_print(A);
	printf("\n\n");

	printf("\nB:\n");
	ArrayPrint(B, ldb, k, N, stdout);

	ArraySetToValue(C, ldc, k, M, 0.0);
	printf("\nC:\n");
	ArrayPrint(C, ldc, k, M, stdout);

	c_dusmm_CSC(order, transpose, k, alpha, A, B, ldb, beta, C, ldc);
	printf("\nC:\n");
	ArrayPrint(C, ldc, k, M, stdout);
	
	return 0;
}
