/* just test csrmm.c */

#include <stdio.h>
#include <malloc.h>
#include "spblas_i.h"
#include "c_blas.h"

/* initialize a matrix */
void ArraySetToValue(double *A, int ldA, int outer , int inner, double value)
{
  double *p, *pend, *t, *tend;

  pend = A + outer*ldA;
  for (p=A; p<pend; p+=ldA)  {
    tend = p+inner;
    for (t=p; t<tend; *t++ = value);
  }
}

/* print a dense matrix */
void ArrayPrint(double *A, int ldA, int outer , int inner, FILE *output)
{
  double *p, *pend, *t, *tend;
  
  fprintf(output, "%d %d\n", outer, inner);

  pend = A + outer*ldA;

  for (p=A; p<pend; p+=ldA) {
    tend = p+inner;
    for (t=p; t<tend; t++) 
      fprintf(output,"%f ",*t);
    fprintf(output,"\n");
  }
}


int main(int argc, char *argv[])
{
	DCSC A = NULL;

	int i;
	int M = 5;
	int N = 5;
	int nz = 15;
	
	double val[] = {1.1, 2.1, 3.1, 4.1, 5.1, 2.2, 3.2, 4.2, 5.2, 3.3, 4.3, 5.3, 4.4, 5.4, 5.5};
	int indx[] =   {0,   1,   2,   3,   4,   1,   2,   3,   4,   2,   3,   4,   3,  4,   4  };
	int pntrb[] = { 0 ,5, 9, 12, 14};
	int pntre[] = { 5, 9, 12,14, 15};


	double UDval[] = {1.1, 1.2, 2.2, 1.3, 2.3, 3.3, 1.4, 2.4, 3.4, 4.4, 1.5, 2.5, 3.5, 4.5, 5.5};
	int UDindx[] =    {0,   0,   1,   0,   1,   2,   0,   1,   2,   3,   0  , 1 ,  2,   3,   4};
	int UDpntrb[] = {  0,  1,  3,  6,  10};
	int UDpntre[] = {  1,  3,  6,  10, 15};

	int prop = sparse_blas_zero_base;
		
	double *B;
	double *UDB;
	int ldb = 5;
	double alpha = 1.0;
	int k = 5;

	int order = blas_colmajor;
	int transpose = blas_no_trans;


	double *C;
	double *UDC;
	int ldc = 5;
	double beta  = 0.0;

	B = (double *) malloc(M * N * sizeof(double));
	
	for (i = 0; i < M * N; i++) {
	  B[i] = 1.0;
	}
	
	C = (double *) malloc(M * N * sizeof(double));
	
	for (i = 0; i < M * N; i++) {
	  C[i] = 0.0;
	}

	UDB = (double *) malloc(M * N * sizeof(double));
	
	for (i = 0; i < M * N; i++) {
	  UDB[i] = 1.0;
	}
	
	UDC = (double *) malloc(M * N * sizeof(double));
	
	for (i = 0; i < M * N; i++) {
	  UDC[i] = 0.0;
	}
	
	A = create_DCSC(M, N, nz, val, indx, pntrb, pntre, prop); 
	DCSC_print(A);
	printf("\n\n");

	printf("\nB_LD:\n");
	ArrayPrint(B, ldb, k, N, stdout);

	c_dussmLD_CSC( order,  transpose, alpha,  A,  k, B, ldb);

	printf("\nB:\n");
	ArrayPrint(B, ldb, k, N, stdout);


	c_dusmm_CSC(order, transpose, k, alpha, A, B, ldb, beta, C, ldc);

	printf("\nC:\n");
	ArrayPrint(C, ldc, k, N, stdout);

	/* upper triangular */

	free(A);

	A = create_DCSC(M, N, nz, UDval, UDindx, UDpntrb, UDpntre, prop); 

	printf("\nUD\nB:\n");
	ArrayPrint(UDB, ldb, k, N, stdout);

	c_dussmUD_CSC( order,  transpose, alpha,  A,  k, UDB, ldb);

	printf("\nB:\n");
	ArrayPrint(UDB, ldb, k, N, stdout);

	/* check with multiply */
	c_dusmm_CSC(order, transpose, k, alpha, A, UDB, ldb, beta, UDC, ldc);

	printf("\nC:\n");
	ArrayPrint(UDC, ldc, k, N, stdout);

	return 0;
}



