#include <stdio.h>
#include <string.h>
#include <malloc.h>
#include "mmio.h"
#include "spblas_i.h"
#include "c_blas.h"
#include "csrmv.h"
#include "cscmv.h"


/* initialize a matrix */
void ArraySetToValue(double *A, int ldA, int outer , int inner, double value)
{
  double *p, *pend, *t, *tend;

  pend = A + outer*ldA;
  for (p=A; p<pend; p+=ldA)  {
    tend = p+inner;
    for (t=p; t<tend; *t++ = value);
  }
}

/* print a dense matrix */
void ArrayPrint(double *A, int ldA, int outer , int inner, FILE *output)
{
  double *p, *pend, *t, *tend;
  
  fprintf(output, "%d %d\n", outer, inner);

  pend = A + outer*ldA;

  for (p=A; p<pend; p+=ldA) {
    tend = p+inner;
    for (t=p; t<tend; t++) 
      fprintf(output,"%f ",*t);
    fprintf(output,"\n");
  }
}

/* save a dense vector */
void ArraySave(double *A, int numRows, char *outFileName)
{
  double *p, *pend;
  int numCols = 1;
  FILE *outFile;

  if ((outFile = fopen(outFileName, "w")) == NULL) 
    exit(1);

  fprintf(outFile,"%d %d\n", numRows, numCols);
  
      pend = A + numRows;
      for (p=A; p<pend; p++) 
	fprintf(outFile,"%f ",*p);
      fprintf(outFile,"\n");
    

  fclose(outFile);  
}


DCSC DCOO_2_DCSC(DCOO A);

int main(int argc, char *argv[]) {



  /*
  DCSC A = NULL;


  double val[] = {1.1, 2.1, 2.2};
  int indx[]  =  { 0,   0,  1};
  int pntrb[] =  { 0,   1 };
  int pntre[] =  { 1,   3 };
  int nz = 3;

  int m = 2;
  int n = 2;

  double y[] = { 2, 2};
  int incy= 1;

  double x[] = { 0, 0};
  int incx = 1;
  
  int prop = sparse_blas_zero_base;
  int transpose = blas_no_trans;

  double alpha = 1.0;
  
  int ind_base = 0;
*/

  /*
  double val[] = {1.1,2.1,2.2,3.1,3.2,3.3};
  int indx[]  =  { 0,  0,  1, 0, 1, 2};
  int pntrb[] =  { 0,  1,  3};
  int pntre[] =  { 1,  3,  6};
  int nz = 6;

  int m = 3;
  int n = 3;

  double y[] = { 1, 1, 1};
  int incy= 1;

  double x[] = { 1, 1, 1};
  int incx = 1;
  
  int prop = sparse_blas_zero_base;
  int transpose = blas_no_trans;

  double alpha = 2.0;
  
  int ind_base = 0;

 
  A = create_DCSC(m, n,  nz, val, indx, 
		pntrb, pntre, prop);

  DCSC_print(A);
  
  printf("\n\n");
  ArrayPrint(x, m, 1, m, stdout);
  
  printf("\n\n");
  c_dussvLD_CSC( transpose, alpha, A, x, incx );

  printf("\n\n");
  ArrayPrint(x, m, 1, m, stdout);
  */



  /* values to create the compressed sparse row matrix A */
    
  SPBLASI_double_CSC CSCA, CSCA1;

  /* values to create the compressed coordinate matrix A and A1*/

  FILE *inX, *f;
  int M, N, nz;   
  int incx = 1;
  int i, j, temp, *I, *J, *I1, *J1;
  double *val;
  int properties = sparse_blas_zero_base;
  int transpose = blas_no_trans;
  SPBLASI_double_COO A;
  
  double alpha = 1.0;
  double *X;
  int k  = 1;
  MM_typecode matcode;
  int ret_code;

  if ((f = fopen("Data/pores_1.mtx", "r")) == NULL) {
    printf("file not found\n");
    exit(1);
  }


  /*
  if ((f = fopen("Data/fidap005.mtx", "r")) == NULL) {
    printf("file not found\n");
    exit(1);
  }
  */
  if ((inX = fopen("Data/X30random", "r")) == NULL) {
    printf("file not found\n");
    exit(1);
  }
  if (mm_read_banner(f, &matcode) != 0)
    {
      printf("Could not process Matrix Market banner.\n");
      exit(1);
    }
  
  /* find out size of sparse matrix .... */
  
  if ((ret_code = mm_read_mtx_crd_size(f, &M, &N, &nz)) !=0)
    exit(1);
  
  /* reserve memory for matrix */
  
  I = (int *) malloc(nz * sizeof(int));
  J = (int *) malloc(nz * sizeof(int));
  val = (double *) malloc(nz * sizeof(double));

  
  /* read the compressed coordinate matrix */
  for (i=0; i<nz; i++)
    {
      fscanf(f, "%d %d %lg\n", &I[i], &J[i], &val[i]);
      /* adjust from 1-based to 0-based */
      
      I[i]--;  
      J[i]--;
    }
  
    if (f !=stdin) fclose(f);
    


    /************************************
      create the structure A (zero-based)
      ***********************************/

    A = create_DCOO(M, N, nz, val, I, J, properties);


    /**************************
      convert the DCOO structure 
      A into the DCSC structure CSCA
      
      *************************/  
    
    CSCA = DCOO_2_DCSC(A);


    /***********************
      allocate memory for X
     **********************/
    
    X = (double *) malloc(N * k * sizeof(double));
    ArraySetToValue(X,M,k,M,1.0);

    printf("\nCSCA:\n");
    DCSC_print_i (CSCA);
    
    printf("\nX:\n");
    ArrayPrint(X, M, k, M, stdout);

    c_dussvLD_CSC( transpose, alpha, CSCA, X, incx );

    printf("\nX:\n");
    ArrayPrint(X, M, k, M, stdout);

    ArraySave(X,M,"Data/outX0");
}
