#include "c_blas.h"
#include "spblas_i.h"
#include "microblas.h"
#include "dcsrmmi.h"
#include "dcscmmi.h"

void c_dusmv_CSR(enum blas_trans_type T, const double alpha, 
		SPBLASI_double_CSR A, const double *x,
		int incx, const double beta, double *y, int incy)
{
	c_dusmm_CSR(blas_colmajor, T, 1, alpha, A, x, incx, beta, y, incy);
}



void c_dusmm_CSR(enum blas_order_type order, enum blas_trans_type T, int k,
		double alpha, SPBLASI_double_CSR A, const double *B,
		int ldB, double beta, double *C, int ldC)
{

	if (T == blas_trans)
		dcscmm(A->prop_, order, A->N_, A->M_, k, alpha, 
			A->pntrb_, A->pntre_, A->val_,
			A->indx_, B, ldB, beta, C, ldC);
	else
		dcsrmm(A->prop_, order, A->M_, A->N_, k, alpha, 
			A->pntrb_, A->pntre_, A->val_,
			A->indx_, B, ldB, beta, C, ldC);
}


void dcsrmm(int Aprop, enum blas_order_type order, 
		int M, int N, int K, double alpha, 
		const int *pntrb, const int *pntre, const double *val,
		const int *indx, const double *B, int ldB, double beta,
		double *C, int ldC)
{
		if (order == blas_colmajor)
		{
			if (Aprop || sparse_blas_one_base)
			{
				B--;
				C--;
			}
			dcsrmm_col(M, K, pntrb, pntre, val, indx,
				alpha, B, ldB, beta, C, ldC);
		}
		else if (order == blas_rowmajor) 
		{
			if (Aprop || sparse_blas_one_base)
			{
				B -= ldB; 
				C -= ldC;
			}
			dcsrmm_row(M, K, pntrb, pntre, val, indx,
				alpha, B, ldB, beta, C, ldC);
		}
			
	
}


/* C = alpha * A*B + beta*C;  A is compressed-row, B&C are row-major  */   

void dcsrmm_row(int M, int K, const int *pntrb, const int *pntre, 
			const double *val, const int *indx, double alpha, 
			const double *B, int ldB, double beta, double *C, int ldC)
{
	int i,j;
	double *Crowi;

	/* requires M*K dot-products */
	for (i=0; i<M; i++)	/* each row of C */
	{
		Crowi = &(C[ldC * i]);
		for (j=0; j<K; j++) /*each column of C*/
		{
			ublas_spdotab(alpha, pntrb[i], pntre[i], val, indx, 
					&B[j], ldB, beta, &(Crowi[j]));
		}
	}
}



/* C = alpha * A*B + beta*y;  A is compressed-row, B&C are col-major  */   

void dcsrmm_col(int M, int K, const int *pntrb, const int *pntre, 
			const double *val, const int *indx, double alpha, 
			const double *B, int ldB, double beta, double *C, int ldC)
{
	int i,k;

	for (i=0; i<K; i++)	/* each column of C */
	{
		const double  *Bi = &B[i*ldB];
		double  *Ci = &C[i*ldC];
		for (k=0; k<M; k++)
		{
			/* could be optimzied to spdotab1 */
			ublas_spdotab(alpha, pntrb[i], pntre[i], val, indx, &B[i], 1,
						beta, &Ci[k]);
		}
	}
}


