#include <stdlib.h>
#include <time.h>
#include <stdio.h>
#include <string.h>
#include <malloc.h>
#include "microblas.h"
#include "c_blas.h"

/*
compile with:
gcc -g  -o microblas_test microblas_test.c microblas.o c_blas.o

*/

/* initialize a matrix */
void ArraySetToValue(double *A, int ldA, int outer , int inner, double value)
{
  double *p, *pend, *t, *tend;

  pend = A + outer*ldA;
  for (p=A; p<pend; p+=ldA)  {
    tend = p+inner;
    for (t=p; t<tend; *t++ = value);
  }
}

/* print a dense matrix */
void ArrayPrint(double *A, int ldA, int outer , int inner, FILE *output)
{
  double *p, *pend, *t, *tend;
  
  fprintf(output, "%d %d\n", outer, inner);

  pend = A + outer*ldA;

  for (p=A; p<pend; p+=ldA) {
    tend = p+inner;
    for (t=p; t<tend; t++) 
      fprintf(output,"%10.10g ",*t);
    fprintf(output,"\n");
  }
}


/*

routines to test these microblas functions:


void ublas_spdotab(double alpha, int ibegin, int iend,
        const double *x, const int *ind, const double *y, int incy,
        double beta, double *r);


void ublas_spdaxpby(int N, int ibegin, int iend, double alpha,
    const double *x, const int *ind, double beta, double *y, int incy);

void ublas_bspdotab_row(int nb, enum blas_trans_type blocktransA,
    int Mb, int k, double alpha,  int ibegin, int iend,
    const double *val, const int *indx,
    const double *B, int ldB, double beta, double *C, int ldC);

void ublas_bspdotab_col(
    int nb, enum blas_trans_type blocktransA,
    int Mb, int k, double alpha,
    int ibegin, int iend, const double *val, const int *indx,
    const double *B, int ldB,
    double beta,
    double *C, int ldC);

 */



int main(int argc, char *argv[]) {
  
  double alpha = 1;
  int ibegin = 0;
  int iend = 20;

  double x[] = {2,2,2,2,2,
		2,2,2,2,2,
		2,2,2,2,2,
		2,2,2,2,2};

  int ind[] = {0,1,2,3,4,
	       5,6,7,8,9,
	       10,11,12,13,14,
	       15,16,17,18,19};

  double y[] = {2,2,2,2,2,
		2,2,2,2,2,
		2,2,2,2,2,
		2,2,2,2,2};


  int incy = 1;
  double beta = 1;
  double r = 2.5;
  
  int N = 20;

  int transpose = blas_no_trans;

  int nb = 2;
  int Mb = 2;
  int k = 2;
  int ldb = 1;
  int ldc = 1;

  double val[] = {1,2,3,4,5,6,7,8,9,10,11,12};
  int indx[] = {1,3,5};
  int pntrb[] = {0};
  int pntre[] = {3};

  double B[] = {1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0};
		  
		
  double C[] = {0,0,0,0};


  /* test ublas_spdotab() */
  ublas_spdotab( alpha,  ibegin,  iend, x, ind, y, incy, beta, &r);
  printf("r = %f\n",r);


  /* test ublas_spdaxpby() */
  ublas_spdaxpby(N, ibegin, iend,  alpha, x, ind, beta, y, incy); 
  ArrayPrint(y, 1, 1 , N, stdout);

  ibegin = pntrb[0];
  iend = pntre[0];

  /*test ublas_bspdotab_row() */
  ublas_bspdotab_row( nb, transpose, Mb, k, alpha,  ibegin, iend, 
		      val, indx, B, ldb, beta, C, ldc);
  printf("\n\n");

  
  ArrayPrint(C,2,2,2,stdout);
}
