#include <stdlib.h>
#include <time.h>
#include <stdio.h>
#include <string.h>
#include <malloc.h>
#include "mmio.h"
#include "spblas_i.h"
#include "c_blas.h"

/*
compile with:
gcc -g -o smalltest3 smalltest3.c spblas_i.o dbsrmm.o microblas.o c_blas.o
 */

/* initialize a matrix */
void ArraySetToValue(double *A, int ldA, int outer , int inner, double value)
{
  double *p, *pend, *t, *tend;

  pend = A + outer*ldA;
  for (p=A; p<pend; p+=ldA)  {
    tend = p+inner;
    for (t=p; t<tend; *t++ = value);
  }
}

/* print a dense matrix */
void ArrayPrint(double *A, int ldA, int outer , int inner, FILE *output)
{
  double *p, *pend, *t, *tend;
  
  fprintf(output, "%d %d\n", outer, inner);

  pend = A + outer*ldA;

  for (p=A; p<pend; p+=ldA) {
    tend = p+inner;
    for (t=p; t<tend; t++) 
      fprintf(output,"%10.10g ",*t);
    fprintf(output,"\n");
  }
}


void BSR_MatMult_CAB_double(
                 const int mb, const int n, const int kb, 
                 const double *val, const int *bindx, 
                 const int *bpntrb, const int *bpntre, const int lb,
                 const double *b, const int ldb,  
                 double *c, const int ldc,
                 const int ind_base)
{
  double t;
  const double *pb;
  double *pc=c;
  const double *pval;
  const double *ptmp;                                  
  int i,j,jb,je,block;
  int cs,bs,br;
  int index;
  int l;                                               
  int ii,jj;
  int m=mb*lb;
  int k=kb*lb;
  int mm=lb*lb;

  bindx-=ind_base;

  for (l=0;l!=n;l++)                            
    for (i=0;i!=m;i++) *pc++ = 0;                  

  for (i=0;i!=mb;i++) {      
    jb = bpntrb[i];             
    je = bpntre[i];             
    cs = i*lb;                  
    for (j=jb;j!=je;j++) {      
      index = bindx[j];
      bs=(index-1)*lb;             
      pval = &val[(j-1)*mm];       
      ptmp = pval;                 
                                   
      pb = &b[bs];                 
      pc = &c[cs];                 
      for (l=0;l!=n;l++) {                      
       for (jj=0;jj!=lb;jj++) {       
         if( pb[jj] != 0.0 ) {           
           t =  pb[jj];            
           for (ii=0;ii!=lb;ii++) {      
             pc[ii] += t* (*pval++);        
           }
         } else {
           pval+=lb;                     
         }
       }
       pb+=ldb; pc+=ldc; pval=ptmp;      
      }                                               
    }
  }
}


int main(int argc, char *argv[])
{
    int ret_code;
    MM_typecode matcode;

    /* values to create the compressed coordinate matrix A */
    int M, N;
    int  nz;
    int nzb;
    int i, j, temp, *I, *J, *I1, *J1;
    int block_ordering;




    int properties = sparse_blas_zero_base;
    int transpose = blas_no_trans;
    int order = blas_colmajor;
    
    /*
    int nb = 2;
    int Mb = 2;
    int Nb = 2;
    int Mnb = nb * Mb;
    int Nnb = nb * Nb;
    int k = 4;
    */

    /*
      the BSR matrix should be 4 blocks of size 2x2
     */
    /*
    int pntrb[] = {0,2};
    int pntre[] = {2,4};
    int indx[] = {0,1,0,1};
    
    double val[] = {1,1,1,1,
		    1,1,1,1,
		    1,1,1,1,
		    1,1,1,1};

    double B[] = {1,0,0,0,
		  0,1,0,0,
		  0,0,1,0,
		  0,0,0,1};
		  
    int ldb = 4;

    double C[] = {0,0,0,0,
		  0,0,0,0,
		  0,0,0,0,
		  0,0,0,0};

    int ldc = 4;
    */

    
    int index_base = 0;

    double ra[]= {1,  0, 0,  0, 1, 0,  0, 0, 1,           /* All of matrix RA */
		  1,  2, 3,  4, 5, 6,  7, 8, 9, 
		  10,11,12,-13,14,15, 16,17,18,
		  1,  0, 0,  0, 1, 0,  0, 0, 1, 
		  1,  4, 7,  2, 5, 8,  3, 6, 9, 
		  1,  0, 0,  0, 1, 0,  0, 0, 1, 
		  1,  0, 0,  0, 1, 0,  0, 0, 1, 
		  1,  0, 0,  0, 1, 0,  0, 0, 1, 
		  -1,  2, 3,  4,-5, 6,  7, 8,-9, 
		  1,  0, 0,  0, 1, 0,  0, 0, 1, 
		  10,-13,16, 11,14,17, 12,15,18,
		  -1,  4, 7,  2,-5, 8,  3, 6,-9,
		  1,  0, 0,  0, 1, 0,  0, 0, 1, 
		  1,  0, 0,  0, 1, 0,  0, 0, 1};

    int rbindx[]={1,2,4,5,1,2,5,3,4,5,1,3,4,5};
    int rbpntrb[]={1,5,8,11};
    int rbpntre[]={5,8,11,15};

    int rmb=4, rkb=5, rlb=3, rm=12, rldb=15, rldc=15;

    double rb[]={1,2,3,4,5,6,7,8,9,10,11,12,1,1,1,
		 1,2,3,4,5,6,7,8,9,10,11,12,1,1,1};
    double rc[]={1,2,3,4,5,6,7,8,9,10,11,12,1,1,1,
		 1,2,3,4,5,6,7,8,9,10,11,12,1,1,1};


    int mb=4, kb=4, lb=3, m=12, ldb=12, ldc=12;


    int  n = 2;


    double alpha = 1;
    double beta = 0;

    /*
    ArrayPrint(B,Mnb,Nnb,ldb,stdout);
    ArrayPrint(C,Mnb,Nnb,ldc,stdout);
    */
    /* 
    BSR_MatMult_CAB_double( rmb, n, rkb, ra, rbindx, rbpntrb, rbpntre, lb, rb, ldb, rc, ldc, index_base);
    */

    dbsrmm( lb,  properties, transpose, order,  rmb,  rkb,  n,  alpha, rbpntrb, rbpntre, ra, rbindx, rb, ldb, beta, rc, ldc);

    ArrayPrint(rc,ldc,2,ldc,stdout);
}



