#include "cblas.h"
#include "spblas_i.h"
#include "spblas.h"


/* could be a macro */
/*
void ArraySetToZero(double *A, int ldA, int outer , int inner)
{
  double *p, *pend, *t, *tend;

  pend = A + outer*ldA;
  for (p=A; p<pend; p+=ldA)
    {
      tend = p+inner;
      for (t=p; t<tend; *t++ = 0.0);
    }
}
*/

/* could be a macro */
/*
void ArrayMultScalar(double *A, int ldA, int outer, int inner, double beta)
{
  double *p, *pend, *t, *tend;

  pend = A + outer*ldA;

  for (p=A; p<pend; p+=ldA)
    {
      tend = p+inner;
      for (t=p; t<tend; *t++ *= beta);
    }
}
*/

void c_dusmm_CSC( enum CBLAS_ORDER order, enum CBLAS_TRANSPOSE T,  int k, const double alpha,
		  SPBLASI_double_CSC A, const double *B, int ldb, const double beta, double *C, int ldc )
{
  const double *val = A->val_;
  const double *pval;
  int *indx = A->indx_;             /* length = nz */
  int *pntrb = A->pntrb_;            /* length = N */
  int *pntre = A->pntre_;            /* length = N */
  int nnz = A->nz_;
  int prop = A->prop_;
  int m = A->M_;
  int n = A->N_;

  int p,i,j,jb,je;
  double t;
  double *c;
  double *pc = C;  
  const double *b;

  SPBLASI_double_CSR CSRA;

  /*
    base one indexing
   */

  if (prop == blas_one_base) {
    if (order == CblasColMajor) {
      c = C - 1;
      b = B - 1;
    }
    else { /* row major base one */
      c = C - ldc;
      b = B - ldb;
    }
  }
  else {
    c = C;
    b = B;
  }


  /* early return for silly argument values */
  if (alpha == 0)
    {
      if (beta != 1.0 && order == CblasColMajor) 
	ArrayMultScalar(C, ldc, k, m, beta);  /* C = beta * C */ 
      else if (beta != 1.0) /* row major */
	ArrayMultScalar(C, ldc, m, k, beta);
      return;
    }
  
  /* handle transpose case using csrmm */
    if (T == CblasTrans)  {

      /* 
	 need to change transpose 
	 to avoid infinite recursion
	 and call c_dusmm_CSR correctly  */
      
      T = CblasNoTrans;
      
      /* create DCSR object */
      CSRA = create_DCSR( m, n, nnz, val, indx, pntrb, pntre, prop);
     
      /* multiply */
      c_dusmm_CSR(order, T, k, alpha, CSRA, B, ldb, beta, C, ldc);
 
      free(CSRA);
      
      return;
    }

  if (order == CblasColMajor)
    {
      if (beta == 0.0) 
	ArraySetToZero(pc, ldc, k, m);
      else if (beta != 1.0)
	ArrayMultScalar(pc, ldc, k, m, beta);


      if (alpha == 1.0) {
	
	for (p = 0; p < k; p++) {                                 
	  for (i = 0; i < n; i++){
	    jb = pntrb[i];
	    je = pntre[i];

	    for (j = jb; j < je; j++)
	      c[indx[j]] += b[i] * val[j];
	  }
	  c += ldc; b += ldb;                                
	}     
	
      }
      else /* alpha != 1.0 */

	for (p = 0; p < k; p++) {                                 
	  for (i = 0; i < n; i++){
	    jb = pntrb[i];
	    je = pntre[i];

	    for (j = jb; j < je; j++)
	      c[indx[j]] += alpha * b[i] * val[j];
	  }
	  c += ldc; b += ldb;                                
	}   

    }

  else if (order == CblasRowMajor)
    {

      if (beta == 0.0) 
	ArraySetToZero(pc, ldc, m, k);
      else if (beta != 1.0)
	ArrayMultScalar(pc, ldc, m, k, beta);

      if (alpha == 1.0)
	
	for (p = 0; p < k; p++) {                                 
	  for (i = 0; i < n; i++){
	    jb = pntrb[i];
	    je = pntre[i];

	    for (j = jb; j < je; j++)
	      c[indx[j] * ldc] += b[i * ldb] * val[j];
	  }
	  c++; b++;                                
	}     
	
      else /* alpha != 1.0 */

	for (p = 0; p < k; p++) {                                 
	  for (i = 0; i < n; i++){
	    jb = pntrb[i];
	    je = pntre[i];

	    for (j = jb; j < je; j++)
	      c[indx[j] * ldc] += alpha * b[i * ldb] * val[j];
	  }
	  c++; b++;                                
	}   

    }
}


