#include "c_blas.h"
#include "spblas_i.h"
#include "csrmm.h"
#include "cscmm.h"

/* could be a macro */
/*
void ArrayMultScalar(double *A, int ldA, int outer , int inner, double beta)
{
  double *p, *pend, *t, *tend;

  pend = A + outer*ldA;

  for (p=A; p<pend; p+=ldA)
    {
      tend = p+inner;
      for (t=p; t<tend; *t++ *= beta);
    }
}
*/

void c_dusmm_CSR( enum blas_order_type order, enum blas_trans_type T,  int k, const double alpha,
		  SPBLASI_double_CSR A, const double *B, int ldb, const double beta, double *C, int ldc )
{
  const double *val = A->val_;
  const double *pval;
  int *indx = A->indx_;             /* length = nz */
  int *pntrb = A->pntrb_;            /* length = M */
  int *pntre = A->pntre_;            /* length = M */
  int nnz = A->nz_;
  int prop = A->prop_;
  int m = A->M_;
  int n = A->N_;

  int p,i,j,jb,je;
  double t;
  double *c;
  const double *b;

  SPBLASI_double_CSC CSCA;

  int index_base = prop || sparse_blas_zero_base;

  /* early return for silly argument values */
  if (alpha == 0)
    {
      if (beta != 1.0 && order == blas_colmajor) 
	ArrayMultScalar(C, ldc, k, m, beta);  /* C = beta * C */ 
      else if (beta != 1.0) /* row major */
	ArrayMultScalar(C, ldc, m, k, beta);
      return;
    }
  
  /* handle transpose case using cscmm */
  
  if (T == blas_trans)  {
    
      T = blas_no_trans;
      
      /* create DCSC object */
      CSCA = create_DCSC( m, n, nnz, val, indx, pntrb, pntre, prop);
      
      /* multiply */
      c_dusmm_CSC(order, T, k, alpha, CSCA, B, ldb, beta, C, ldc);
      
      free(CSCA);
      return;
  }

  /*
    base one indexing
    */
  
  if (prop == sparse_blas_one_base) 
    if (order == blas_colmajor) 
      b = B - 1;
    else  /* row major base one */
      b = B - ldb;
  else 
    b = B;

  c = C;

  

  if (order == blas_colmajor)
    {
      if (alpha == 1.0)
	for (p = 0; p < k; p++)
	  {
	    pval = val;
	    for (i = 0; i < m; i++) {
	      if (beta == 0) 
		t = 0;
	      else if (beta == 1.0)
		t = c[i];
	      else /* beta != 1.0 */
		t = beta * c[i];

	      jb = pntrb[i] - index_base;
	      je = pntre[i] - index_base;
	      
	      for (j = jb; j < je; j++)
		t += b[(indx[j])] * (*pval++);
	      c[i] = t;
	    }      
	    c += ldc; b+=ldb;
	  }
          
      else /* alpha != 1.0 */
	for (p = 0; p < k; p++)
	  {
	    pval = val;
	    for (i = 0; i < m; i++) {
	      if (beta == 0) 
		t = 0;
	      else if (beta == 1.0)
		t = c[i];
	      else /* beta != 1.0 */
		t = beta * c[i];

	      jb = pntrb[i] - index_base;
	      je = pntre[i] - index_base;
	      
	      for (j = jb; j < je; j++)
		t += alpha * b[indx[j]] * (*pval++);
	      c[i] = t;
	    }      
	    c += ldc; b+=ldb;
	  }
    }

  else if (order == blas_rowmajor)
    {
      if (alpha == 1.0)
	
	for (p=0; p < k; p++)
	  {
	    pval = val;
	    for (i = 0; i < m; i++) {
	      if (beta == 0) 
		t = 0;
	      else if (beta == 1.0)
		t = c[(i * ldc)];
	      else  /* beta != 1.0 */
		t = beta * c[(i * ldc)];
	      jb = pntrb[i] - index_base;
	      je = pntre[i] - index_base;
	      
	      for (j = jb; j < je; j++)
		t += b[(indx[j] * ldb)] * (*pval++);
	      c[(i * ldc)] = t;
	    }      
	    c++; b++;
	  }
      
      else 
	for (p=0; p < k; p++)
	  {
	    pval = val;
	    for (i = 0; i < m; i++) {
	      if (beta == 0) 
		t = 0;
	      else if (beta == 1.0)
		t = c[(i * ldc)];	      
	      else  /* beta != 1.0 */
		t = beta * c[(i * ldc)];
	      jb = pntrb[i] - index_base;
	      je = pntre[i] - index_base;
	      
	      for (j = jb; j < je; j++)
		t += alpha * b[(indx[j] * ldb)] * (*pval++);
	      c[(i * ldc)] = t;
	    }      
	    c++; b++;
	  }
    }
}


