/* just test csrmm.c */

#include <stdio.h>
#include <malloc.h>
#include "spblas_i.h"
#include "c_blas.h"

/* initialize a matrix */
void ArraySetToValue(double *A, int ldA, int outer , int inner, double value)
{
  double *p, *pend, *t, *tend;

  pend = A + outer*ldA;
  for (p=A; p<pend; p+=ldA)  {
    tend = p+inner;
    for (t=p; t<tend; *t++ = value);
  }
}

/* print a dense matrix */
void ArrayPrint(double *A, int ldA, int outer , int inner, FILE *output)
{
  double *p, *pend, *t, *tend;
  
  fprintf(output, "%d %d\n", outer, inner);

  pend = A + outer*ldA;

  for (p=A; p<pend; p+=ldA) {
    tend = p+inner;
    for (t=p; t<tend; t++) 
      fprintf(output,"%f ",*t);
    fprintf(output,"\n");
  }
}

int main(int argc, char *argv[])
{
	DCSR A = NULL;

	int M = 5;
	int N = 5;
	int nz = 15;
	double val[] = {1.1, 2.1, 2.2, 3.1, 3.2, 3.3, 4.1, 4.2, 4.3, 4.4, 5.1, 5.2, 5.3, 5.4, 5.5};
	int indx[]  =  { 0,   0,   1,   0,   1,   2,   0,   1,   2,   3,   0  , 1 ,  2,   3,   4};
	int pntrb[] =  { 0,  1,  3,  6,  10};
	int pntre[] =  { 1,  3,  6,  10, 15};

	double UDval[] = {1.1, 1.2, 1.3, 1.4, 1.5, 2.2, 2.3, 2.4, 2.5, 3.3, 3.4, 3.5, 4.4, 4.5, 5.5};
	int UDindx[]  ={0,   1,   2,   3,   4,   1,   2,   3,   4,   2,   3,   4,   3,   4,   4 };
	int UDpntrb[] ={0, 5, 9,  12,  14  };
	int UDpntre[] ={5, 9, 12, 14, 15 };
	double UDB[5][5] = {{1.0, 1.0, 1.0, 1.0, 1.0},
			    {1.0, 1.0, 1.0, 1.0, 1.0},
			    {1.0, 1.0, 1.0, 1.0, 1.0},
			    {1.0, 1.0, 1.0, 1.0, 1.0},
			    {1.0, 1.0, 1.0, 1.0, 1.0}};
	
	double UDC[5][5] = {{0.0, 0.0, 0.0, 0.0, 0.0},
			    {0.0, 0.0, 0.0, 0.0, 0.0},
			    {0.0, 0.0, 0.0, 0.0, 0.0},
			    {0.0, 0.0, 0.0, 0.0, 0.0},
			    {0.0, 0.0, 0.0, 0.0, 0.0}};
	

	int prop = sparse_blas_zero_base;
	
	double B[5][5] = {{1.0, 1.0, 1.0, 1.0, 1.0},
			  {1.0, 1.0, 1.0, 1.0, 1.0},
			  {1.0, 1.0, 1.0, 1.0, 1.0},
			  {1.0, 1.0, 1.0, 1.0, 1.0},
			  {1.0, 1.0, 1.0, 1.0, 1.0}};

	int ldb = 5;
	double alpha = 1.0;
	int k = 5;

	int order = blas_colmajor;
	int transpose = blas_no_trans;

	double beta = 0.0;
	double C[5][5] = {0};
	int ldc = 5;

	A = create_DCSR(M, N, nz, val, indx, pntrb, pntre, prop); 
	DCSR_print(A);
	printf("\n\n");

	printf("\nB:\n");
	ArrayPrint(B, ldb, k, N, stdout);


	c_dussmLD_CSR( order,  transpose, alpha,  A,  k, B, ldb);

	printf("\nB:\n");
	ArrayPrint(B, ldb, k, N, stdout);

	/* now, multiply C by A to check the results */

	c_dusmm_CSR(order, transpose, k, alpha, A, B, ldb, beta, C, ldc);

	printf("\nC:\n");
	ArrayPrint(C, ldc, k, N, stdout);


	/* check upper triangular case */

	free(A);


	A = create_DCSR(M, N, nz, UDval, UDindx, UDpntrb, UDpntre, prop); 
	DCSR_print(A);
	printf("\n\n");

	printf("\nUD\n B:\n");
	ArrayPrint(UDB, ldb, k, N, stdout);


	c_dussmUD_CSR( order,  transpose, alpha,  A,  k, UDB, ldb);

	printf("\nB:\n");
	ArrayPrint(UDB, ldb, k, N, stdout);

	/* now, multiply C by A to check the results */

	c_dusmm_CSR(order, transpose, k, alpha, A, UDB, ldb, beta, UDC, ldc);

	printf("\nC:\n");
	ArrayPrint(UDC, ldc, k, N, stdout);

	return 0;
}
