/*  csrmv.c  (Wed Aug  4 15:47:25 EDT 1999)
 *  Authors:  
 *  Galen Wilkerson and Roldan Pozo
 *  National Institute of Standards and Technology
 *
 *  Contact:  galen@cam.nist.gov or pozo@cam.nist.gov
 */

#include <stdio.h>
#include "spblas_i.h"
#include "cblas.h"
#include "spblas.h"
#include "csrmv.h"

void c_dusmv_CSR( enum CBLAS_TRANSPOSE T,  double alpha,
		  SPBLASI_double_CSR A, double *x, int incx,  double beta, double *y, int incy )
{
  /********************************
    y <-  alpha * A * x +  beta * y
    ******************************/

  double *val = A->val_;
  int *indx = A->indx_;             /* length = nz */
  int *pntrb = A->pntrb_;            /* length = M */
  int *pntre = A->pntre_;            /* length = M */
  int nnz = A->nz_;
  int prop = A->prop_;
  int m = A->M_;
  int n = A->N_;

  int i, j, jb, je; 
  double t;
  double *py=y;
  double *pval;
  double z; 
  double valtmp;

  SPBLASI_double_CSC CSCA;

  int index_base = prop || blas_zero_base;

  /* handle transpose case using cscmv */
  if (T == CblasTrans)  {

      T = CblasNoTrans;
      /* create DCSC object */
      CSCA = create_DCSC( m, n, nnz, val, indx, pntrb, pntre, prop);

      /* multiply */
      c_dusmv_CSC( T, alpha, CSCA, x, incx, beta, y, incy );

      free(CSCA);
      return;
  }
  
  if (incy == 1 && incx ==1 )
    {
      if (prop == blas_one_base)
	x--;
      
      for (i=0;i!=m;i++) *py++ *= beta;                 
      
      pval = val;
      for (i=0;i!=m;i++) {
	t = 0;
	jb = pntrb[i] - index_base;
	je = pntre[i] - index_base;
	for (j=jb;j!=je;j++)
	  t += alpha * x[indx[j]] * (*pval++);
	y[i] += t;
      }
    }
  else
    {
      if (prop == blas_one_base) 
	x-=incx;
      
      for (i=0;i!=m;i++) { 
	*py *= beta;                 
	py += incy;
      }

      pval = val;
      for (i=0;i!=m;i++) {
	t = 0;
	jb = pntrb[i] - index_base;
	je = pntre[i] - index_base;
	for (j=jb;j!=je;j++)
	  t += alpha * x[indx[j] * incx] * (*pval++);
	y[i * incy] += t;
      }
    }
}







