#include <stdio.h>
#include <string.h>
#include "c_blas.h"
#include "spblas_i.h"
#include ""
#include "csrsv.h"


void c_dussvLD_CSC( 
		   enum blas_trans_type T, 
		   const double alpha, 
		   SPBLASI_double_CSC A, 
		   double *x, 
		   int incx )
     
{
  double *val = A->val_;
  int *indx = A->indx_;             /* length = nz */
  int *pntrb = A->pntrb_;            /* length = M */
  int *pntre = A->pntre_;            /* length = M */
  int nnz = A->nz_;
  int prop = A->prop_;
  int m = A->M_;
  int n = A->N_;

  int i, j, jb, je, index;
  double *px=x;
  double z; 
  double valtmp;

  SPBLASI_double_CSR CSRA;

  /* handle transpose case using csrsv */
  if (T == blas_trans)  {

      T = blas_no_trans;
      
      /* create DCSR object */
      CSRA = create_DCSR( m, n, nnz, val, indx, pntrb, pntre, prop);
      
      /* triangular solve */
      c_dussvUD_CSR( T, alpha, CSRA, x, incx);

      free(CSRA);
      
      return;
  }
  

  /* early return for silly argument values */
  if (alpha == 0)
    {
      for (i = 0; i < m; i++) 
	x[i] = 0.0;
      return;
    }


  if (prop == sparse_blas_one_base) {
    x -= incx;
    px = x;
  }

 
  px=x;
  for (i=0;i!=m;i++) {
    jb = pntrb[i];
    je = pntre[i];
    z =  px[i * incx] / val[jb];
    px[i * incx] = z;
    for (j=jb+1;j<je;j++) {
      x[indx[j] * incx] -= z*val[j];
    }
  }
  if (alpha != 1.0) 
    for (i=0;i!=m;i++){
      *px *= alpha ;            
      px+=incx;                             
    }       
}



void c_dussvLU_CSC( 
		   enum blas_trans_type T, 
		   const double alpha, 
		   SPBLASI_double_CSC A, 
		   double *x, 
		   int incx )
     
{
  double *val = A->val_;
  int *indx = A->indx_;             /* length = nz */
  int *pntrb = A->pntrb_;            /* length = M */
  int *pntre = A->pntre_;            /* length = M */
  int nnz = A->nz_;
  int prop = A->prop_;
  int m = A->M_;
  int n = A->N_;

  int i, j, jb, je, index;
  double *px=x;
  double z; 

  SPBLASI_double_CSR CSRA;

  /* handle transpose case using csrsv */
  if (T == blas_trans)  {

      T = blas_no_trans;
      
      /* create DCSR object */
      CSRA = create_DCSR( m, n, nnz, val, indx, pntrb, pntre, prop);
      
      /* triangular solve */
      c_dussvUU_CSR( T, alpha, CSRA, x, incx);
 
      free(CSRA);
      
      return;
  }
  

  /* early return for silly argument values */
  if (alpha == 0)
    {
      for (i = 0; i < m; i++) 
	x[i] = 0.0;
      return;
    }


  if (prop == sparse_blas_one_base) {
    x = x - incx;
    px = x;
  }

 
  px=x;
  for (i=0;i!=m;i++) {
    jb = pntrb[i];
    je = pntre[i];
    z =  px[i * incx];
    for (j=jb;j<je;j++) {
      x[indx[j] * incx] -= z*val[j];
    }
  }
  if (alpha != 1.0) 
    for (i=0;i!=m;i++){
      *px *= alpha ;            
      px+=incx;                             
    }       
}



void c_dussvUD_CSC(enum blas_trans_type T, 
		   const double alpha, 
		   SPBLASI_double_CSC A, 
		   double *x, 
		   int incx )
     
{
  double *val = A->val_;
  int *indx = A->indx_;             /* length = nz */
  int *pntrb = A->pntrb_;            /* length = M */
  int *pntre = A->pntre_;            /* length = M */
  int nnz = A->nz_;
  int prop = A->prop_;
  int m = A->M_;
  int n = A->N_;

  int i, j, jb, je, index;
  double *px=x;
  double z; 
  double valtmp;

  SPBLASI_double_CSR CSRA;


  /* handle transpose case using csrsv */
  if (T == blas_trans)  {

      T = blas_no_trans;
      
      /* create DCSR object */
      CSRA = create_DCSR( m, n, nnz, val, indx, pntrb, pntre, prop);
      
      /* triangular solve */
      c_dussvLD_CSR( T, alpha, CSRA, x, incx);

      free(CSRA);
      
      return;
  }

  /* early return for silly argument values */
  if (alpha == 0)
    {
      for (i = 0; i < m; i++) 
	x[i * incx] = 0.0;
      return;
    }

  if (prop == sparse_blas_one_base) {
    x = x - incx;
    px = x;
  }

  px=x;
  for (i=m-1;i!=-1;i--) {
    jb = pntrb[i];
    je = pntre[i]-1;
    z = px[i * incx] /val[je];
    px[i * incx] = z;
    for (j=jb;j<je;j++) {
      x[indx[j]* incx] -= z * val[j];
    }
  }
  if (alpha != 1.0)
    for (i=0;i!=m;i++) {                  
      *px *= alpha ;            
      px+=incx;                             
    }       

}




void c_dussvUU_CSC(enum blas_trans_type T, 
		   const double alpha, 
		   SPBLASI_double_CSC A, 
		   double *x, 
		   int incx )
     
{
  double *val = A->val_;
  int *indx = A->indx_;             /* length = nz */
  int *pntrb = A->pntrb_;            /* length = M */
  int *pntre = A->pntre_;            /* length = M */
  int nnz = A->nz_;
  int prop = A->prop_;
  int m = A->M_;
  int n = A->N_;

  int i, j, jb, je, index;
  double *px=x;
  double z; 

  SPBLASI_double_CSR CSRA;


  /* handle transpose case using csrsv */
  if (T == blas_trans)  {

      T = blas_no_trans;
      
      /* create DCSR object */
      CSRA = create_DCSR( m, n, nnz, val, indx, pntrb, pntre, prop);
      
      /* triangular solve */
      c_dussvLU_CSR( T, alpha, CSRA, x, incx);

      free(CSRA);
      
      return;
  }

  /* early return for silly argument values */
  if (alpha == 0)
    {
      for (i = 0; i < m; i++) 
	x[i * incx] = 0.0;
      return;
    }

  if (prop == sparse_blas_one_base) {
    x = x - incx;
    px = x;
  }

  px=x;
  for (i=m-1;i!=-1;i--) {
    jb = pntrb[i];
    je = pntre[i];
    z = px[i * incx];
    for (j=jb;j<je;j++) {
      x[indx[j]* incx] -= z * val[j];
    }
  }
  if (alpha != 1.0)
    for (i=0;i!=m;i++) {                  
      *px *= alpha ;            
      px+=incx;                             
    }       

}
