#ifndef SPBLAS_I_H
#define SBPLAS_I_H

#ifndef NULL
#define NULL 0
#endif

#define DCOO SPBLASI_double_COO
#define DCSR SPBLASI_double_CSR
#define DCSC SPBLASI_double_CSC
#define DDIA SPBLASI_double_DIA

/* Coordinate storage format */
typedef struct
{
    int M_, N_, nz_;
    int prop_;
    double *val_;           /* length = nz */
    int *i_;                /* length = nz */
    int *j_;                /* length = nz */
    int reclaim_space;      /* boolean, true if mallocs own val, i, j */    
} *DCOO, DCOO_struct;


DCOO create_DCOO( int M, int N, int nz, double *val, int *indx, int 
        *jndx, int properties);
DCOO create_DCOO_copy( int M, int N, int nz, double *val,
    int *indx, int *jndx, int properties);
void destroy_DCOO( DCOO A);

void DCOO_print(DCOO A);



/* Block-Coordinate storage format */
typedef struct
{
    int nb_;                /* blocksize (nb x nb) */
    int Mb_, Nb_, nzb_;     /* number of blocks */
    int block_ordering_;    /* 0 if row, 1 if column */

    int M_, N_, nz_;        /* M = Mb*nb, N = Nb*nb, nz = nzb*nb*nb */
    int prop_;
    double *val_;           /* length = nz  (nzb*nb*nb) */
    int *i_;                /* length = nzb */
    int *j_;                /* length = nzb */
    int reclaim_space;      /* boolean, true if mallocs own val, i, j */    
} *DBCO, DBCO_struct;


DBCO create_DBCO( int nb, int M, int N, int nz, double *val, int *indx, int 
        *jndx, int properties);
DBCO create_DBCO_copy( int nb, int M, int N, int nz, double *val,
    int *indx, int *jndx, int properties);
void destroy_DBCO( DBCO A);
void DBCO_print(DBCO A);


             /* ------------------------------- * /


/* Compressed-Row storage format: Note that pntrb[] and pntre[]
    can overlap*/

typedef struct
{
    int M_, N_, nz_;
    int prop_;
    double *val_;           /* length = nz */
    int *indx_;             /* length = nz */
    int *pntrb_;            /* length = M */
    int *pntre_;            /* length = M */
    int reclaim_space;      /* boolean, true if mallocs own val, i, j */    
} *DCSR, DCSR_struct;


DCSR create_DCSR( int M, int N, int nz, double *val, int *indx, 
    int *pntrb, int *pntre, int properties);
DCSR create_DCSR_copy( int M, int N, int nz, double *val, int *indx, 
    int *pntrb, int *pntre, int properties);
void destroy_DCSR( DCSR A);
void DCSR_print(DCSR A);
void DCSR_print_i(DCSR A);


/* Block Compressed-Row storage format */

typedef struct
{
    int nb_;                /* blocksize (nb x nb) */
    int Mb_, Nb_, nzb_;     /* number of blocks */
    int block_ordering_;    /* 0 if row, 1 if column */

    int M_, N_, nz_;
    int prop_;
    double *val_;           /* length = nz */
    int *indx_;             /* length = nzb */
    int *pntrb_;            /* length = Mb */
    int *pntre_;            /* length = Mb */
    int reclaim_space;      /* boolean, true if mallocs own val, i, j */    
} *DBSR, DBSR_struct;


DBSR create_DBSR( int nb, int M, int N, int nz, double *val, int *indx, 
    int *pntrb, int *pntre, int properties);
DBSR create_DBSR_copy( int nb, int M, int N, int nz, double *val, int *indx, 
    int *pntrb, int *pntre, int properties);
void destroy_DBSR( DBSR A);
void DBSR_print(DBSR A);
void DBSR_print_i(DBSR A);


             /* ------------------------------- * /


/* Compressed-Col storage format: Note that pntrb[] and pntre[]
    can overlap*/

typedef struct
{
    int M_, N_, nz_;
    int prop_;
    double *val_;           /* length = nz */
    int *indx_;             /* length = nz */
    int *pntrb_;            /* length = N */
    int *pntre_;            /* length = N */
    int reclaim_space;      /* boolean, true if mallocs own val, i, j */    
} *DCSC, DCSC_struct;


DCSC create_DCSC( int M, int N, int nz, double *val, int *indx, 
    int *pntrb, int *pntre, int properties);
DCSC create_DCSC_copy( int M, int N, int nz, double *val, int *indx, 
    int *pntrb, int *pntre, int properties);
void destroy_DCSC( DCSC A);


typedef struct
{
    int nb_;                /* blocksize (nb x nb) */
    int Mb_, Nb_, nzb_;     /* number of blocks */
    int block_ordering_;    /* 0 if row, 1 if column */

    int M_, N_, nz_;        /* M = Mb*nb, N=Nb*nb, nz = nzb*nb*nb */
    int prop_;
    double *val_;           /* length = nz */
    int *indx_;             /* length = nzb */
    int *pntrb_;            /* length = Nb */
    int *pntre_;            /* length = Nb */
    int reclaim_space;      /* boolean, true if mallocs own val, i, j */    
} *DBSC, DBSC_struct;


DBSC create_DBSC( int M, int N, int nz, double *val, int *indx, 
    int *pntrb, int *pntre, int properties);
DBSC create_DBSC_copy( int M, int N, int nz, double *val, int *indx, 
    int *pntrb, int *pntre, int properties);
void destroy_DBSC( DBSC A);




             /* ------------------------------- * /

/* Generic Storage Matrix: entries in global table */

enum SPBLASI_spmatformat { DCOO_typecode, DCSC_typecode, DSCR_typecode,
                   DDIA_typecode};

typedef struct
{
    enum SPBLASI_spmatformat f_;
    union
    {
        DCOO dcoo_;
        DCSR dcsr_;
        DCSC dcsc_;
        DBCO dbco_;
        DBSR dbsr_;
        DBSC dbsc_;
    }mat;
} SPBLASI_GenericSpMat;

#endif
