/*  test_vect_mult.c  (Wed Aug  4 15:47:25 EDT 1999)
 *  Author:  Galen Wilkerson
 *
 *  See README.txt for usage
 *
 *  Significant parts of this code modified from example_read.c
 *  by Roldan Pozo and Karin Remington
 *
 *  Contact:  galen@cam.nist.gov or pozo@cam.nist.gov
 */


#include <stdio.h>
#include <malloc.h>
#include "mmio.h"
#include "spblas_i.h"
#include "spblas.h"
#include "cblas.h"


/* all these functions could be macros */
/* initialize a matrix */
void ArraySetToValue(double *A, int ldA, int outer , int inner, double value)
{
  double *p, *pend, *t, *tend;

  pend = A + outer*ldA;
  for (p=A; p<pend; p+=ldA)  {
    tend = p+inner;
    for (t=p; t<tend; *t++ = value);
  }
}


int read_mtx_size(FILE *f, int *M, int *N, int *nz )
{
    char line[MM_MAX_LINE_LENGTH];
    int num_items_read;

    /* set return null parameter values, in case we exit with errors */
    *M = *N = *nz = 0;

    /* now continue scanning until you reach the end-of-comments */
    do 
    {
        if (fgets(line,MM_MAX_LINE_LENGTH,f) == NULL) 
            return MM_PREMATURE_EOF;
    }while (line[0] == '%');

    /* line[] is either blank or has M,N, nz */
    if (sscanf(line, "%d %d %d", M, N, nz) == 3)
        return 0;
        
    else
    do
    { 
        num_items_read = fscanf(f, "%d %d %d", M, N, nz); 
        if (num_items_read == EOF) return MM_PREMATURE_EOF;
    }
    while (num_items_read != 3);

    return 0;
}


int main(int argc, char *argv[])
{
    int ret_code;
    MM_typecode matcode;
    FILE *f, *inX, *inX1, *outY, *outY1;
    int M, N, nb, nz;   
    int i, *I, *J, *I1, *J1;
    double *val;
    int properties = blas_zero_base;
    int transpose = CblasNoTrans;

    SPBLASI_double_COO A, A1;
    double alpha = 1.0;
    double *x, *x1;
    int incx = 1;
    int incx1 = 2;

    double beta = 0.0;
    double *y, *y1;
    int incy = 1;
    int incy1 = 1;

    if (argc < 6)
	{
	  fprintf(stderr, "Usage: %s [martix-market-filename] [x-vector filename] [y-vector filename] [x1-vector filename] [y1-vector filename]\n", argv[0]);
	  exit(1);
	}
    else    
    { 
      if ((f = fopen(argv[1], "r")) == NULL) 
	exit(1);
      if ((inX = fopen(argv[2], "r")) == NULL) 
	exit(1);
      if ((outY = fopen(argv[3], "w")) == NULL) 
	exit(1);
      if ((inX1 = fopen(argv[4], "r")) == NULL) 
	exit(1);
      if ((outY1 = fopen(argv[5], "w")) == NULL) 
	exit(1);
    }


    /* find out size of sparse matrix .... */

    if ((ret_code = read_mtx_size(f, &M, &N, &nb, &nz)) !=0)
        exit(1);


    /* reserve memory for matrices */

    I = (int *) malloc(nz * sizeof(int));
    J = (int *) malloc(nz * sizeof(int));
    I1 = (int *) malloc(nz * sizeof(int));
    J1 = (int *) malloc(nz * sizeof(int));
    val = (double *) malloc(nb * nb * nz * sizeof(double));


    /* NOTE: when reading in doubles, ANSI C requires the use of the "l"  */
    /*   specifier as in "%lg", "%lf", "%le", otherwise errors will occur */
    /*  (ANSI C X3.159-1989, Sec. 4.9.6.2, p. 136 lines 13-15)            */

    /* read the compressed coordinate matrix */
    for (i=0; i<nz; i++)
      {
        fscanf(f, "%d %d %lg\n", &I1[i], &J1[i], &val[i]);
        /* adjust from 1-based to 0-based */
	I[i] = I1[i] - 1;  
	J[i] = J1[i] - 1;
      }


    /* read the nonzero values */
    for (i=0; i < nb * nb * nz; i++)
      fscanf(f, "%lg\n", &val[i]);
    
    if (f !=stdin) fclose(f);

    /************************
      create the structure A
      read the vector x
      create the vector y
    ************************/
    create_DBCO(nb, M, N, nz, val, I, J, properties);


    /************************/
    /* now write out matrix */
    /************************/

    DBCO_print(A);

    x = (double *) malloc(incx * N * sizeof(double));
    x1 = (double *) malloc(incx1 * N * sizeof(double));
 

    /**********************
      read the x and x1-vector
      **********************/
    
    printf("\n x= \n");
    for (i = 0; i < ( incx * N ); i++) {
      fscanf(inX, "%lg\n", &x[i]);
      fprintf(stdout, "%f\n", x[i]);
    }

    printf("\n x1= \n");
    for (i = 0; i < ( incx1 * N ); i++) {
      fscanf(inX1, "%lg\n", &x1[i]);
      fprintf(stdout, "%f\n", x1[i]);
    }
    
    y = (double *) malloc(incy * M * sizeof(double));
    y1 = (double *) malloc(incy1 * M * sizeof(double));


    /****************************
      Case 0
      transpose = no trans
      alpha = 3.25
      nb = <small>
      nb_ordering = col
      beta = -2.75
      incx = 1
      incy = 1
      prop = base 0
      order = col

      ****************************/
    
    /******************
      initialize y
      ******************/

    ArraySetToValue(y, M, 1 ,M, 0.0);



    /********************************************
      multiply the matrix A by x, a vector of 1's
      y = A*x
      
      ********************************************/
    
    /*
    c_dusmv_COO(transpose, alpha, A, x, incx, beta, y, incy);
    */


    /********************
      print the vector y
      and save it to file
      *********************/
    
    printf("\ny = \n");
     
    for (i=0; i< (incy * M); i++) {
      /*
	fprintf(stdout, "%f\n", *(y+i));
	*/
      fprintf(outY, "%f\n", *(y+i));
    }


    /****************************
      Case 1
      transpose = trans
      alpha = 3.25
      nb = <large>
      nb_ordering = row
      beta = -2.75
      incx = 2
      incy = 1
      prop = base 1
      order = row

      ****************************/
    
    transpose = CblasTrans;

     /******************
      initialize y1
      ******************/
    
    
    ArraySetToValue(y1, M, 1 ,M, 0.0);

     
    /*
    c_dusmv_COO(transpose, alpha, A1, x1, incx1, beta, y1, incy1);
    */


    /********************
      print the vector y1
      and save it to file
      *********************/
    
    printf("\ny1 = \n");
     
    for (i=0; i< (incy1 * M); i++) {
      /*
	fprintf(stdout, "%f\n", *(y1+i));
	*/
      fprintf(outY1, "%f\n", *(y1+i));
    }


    /* free memory */
    free(I);
    free(I1);
    free(J);
    free(J1);
    free(val);
    free(x);
    free(x1);
    free(y);
    free(y1);

    destroy_DCOO(A);
    destroy_DCOO(A1);


    return 0;
}
