/* c_blas.h: bindings for the offical BLAS standard */

#ifndef C_BLAS_H
#define C_BLAS_H

enum blas_cmach_type {
            blas_base      = 0x4000,
            blas_t         = 0x4001,
            blas_rnd       = 0x4002,
            blas_ieee      = 0x4003,
            blas_emin      = 0x4004,
            blas_emax      = 0x4005,
            blas_eps       = 0x4006,
            blas_prec      = 0x4007,
            blas_underflow = 0x4008,
            blas_overflow  = 0x4009,
            blas_sfmin     = 0x400A };
enum blas_norm_type {
            blas_one_norm       = 0x4100,
            blas_real_one_norm  = 0x4101,
            blas_two_norm       = 0x4102,
            blas_frobenius_norm = 0x4103,
            blas_inf_norm       = 0x4104,
            blas_real_inf_norm  = 0x4105,
            blas_max_norm       = 0x4106,
            blas_real_max_norm  = 0x4107 };
enum blas_sort_type { 
            blas_increasing_order = 0x4200,
            blas_decreasing_order = 0x4201 };
enum blas_side_type {
            blas_left_side =  0x4300,
            blas_right_side = 0x4301 };
enum blas_trans_type {
            blas_no_trans   = 0x4400,
            blas_trans      = 0x4401,
            blas_conj_trans = 0x4402 };
enum blas_conj_type {
            blas_conj    = 0x4500,
            blas_no_conj = 0x4501 };
enum blas_jrot_type {
            blas_jrot_inner  = 0x4600,
            blas_jrot_outer  = 0x4601,
            blas_jrot_sorted = 0x4602 };
enum blas_prec_type {
            blas_single     = 0x4700,
            blas_double     = 0x4701,
            blas_indigenous = 0x4702,
            blas_extra      = 0x4703 };
enum blas_order_type {
            blas_rowmajor = 0x0002,
            blas_colmajor = 0x0004 };
enum blas_diag_type {
            blas_non_unit_diag = 0x0008,
            blas_unit_diag     = 0x0010 };
enum blas_uplo_type  {
            blas_upper = 0x0020,
            blas_lower = 0x0040 };

 /* Matrix properties */

enum sparse_blas_order_type {
            sparse_blas_rowmajor = 0x0002,
            sparse_blas_colmajor = 0x0004 };
enum sparse_blas_base_type {
            sparse_blas_zero_base = 0x0000,
            sparse_blas_one_base  = 0x0001 };
enum sparse_blas_diag_type {
            sparse_blas_non_unit_diag = 0x0008,
            sparse_blas_unit_diag     = 0x0010 };
enum sparse_blas_uplo_type  {
            sparse_blas_upper = 0x0020,
            sparse_blas_lower = 0x0040 };
enum sparse_blas_symmetry_type {
            sparse_blas_general   = 0x0080,
            sparse_blas_symmetric = 0x0100,
            sparse_blas_hermitian = 0x0110 };
enum sparse_blas_repeated_indices_type {
            sparse_blas_no_repeated_indices = 0x1200,
            sparse_blas_repeated_indices    = 0x1400 };




void c_dgemm( 
		enum blas_order_type order, 
		enum blas_trans_type transa,
		enum blas_trans_type transb,
		int m, int n, int k, double alpha, const double A[],
		int lda, const double B[], int ldB, double beta,
		double C[], int ldC);


void c_dgemv( 
		enum blas_order_type order, 
		enum blas_trans_type transa,
		int m, int n, double alpha, const double A[],
		int lda, const double x[], int incx, double beta,
		double y[], int incy);

#endif

