#include "c_blas.h"
#include "spblas_i.h"

/* could be a macro */
void ArraySetToZero(double *A, int ldA, int outer , int inner)
{
  double *p, *pend, *t, *tend;

  pend = A + outer*ldA;
  for (p=A; p<pend; p+=ldA)
    {
      tend = p+inner;
      for (t=p; t<tend; *t++ = 0.0);
    }
}

	
void ArrayMultScalar(double *A, int ldA, int outer , int inner, double beta)
{
  double *p, *pend, *t, *tend;

  pend = A + outer*ldA;

  for (p=A; p<pend; p+=ldA)
    {
      tend = p+inner;
      for (t=p; t<tend; *t++ *= beta);
    }
}


void c_dusmm_COO( enum blas_order_type order, enum blas_trans_type T,  int k, const double alpha,
		  SPBLASI_double_COO A, const double *B, int ldb, const double beta, double *C, int ldc )
{
  const double *val = A->val_;
  const int *I = A->i_;
  const int *J = A->j_;
  int nnz = A->nz_;
  int prop = A->prop_;
  int m = A->M_;
  int n = A->N_;

  int i,p;

  double *c;
  const double *b;

  if (prop == sparse_blas_one_base) {
    c = C - 1;
    b = B - 1;
  }
  else {
    c = C;
    b = B;
  }

  /* early return for silly argument values */
  if (alpha == 0)
    {
      if ((beta != 1.0) && (order == blas_colmajor))
	ArrayMultScalar(C, ldc, k, m, beta);  /* C = beta * C */ 
      else if (beta != 1.0) /* row major */
	ArrayMultScalar(C, ldc, m, k, beta);  
      return;
    }

  if (T == blas_trans) /* swap I[] and J[] */
    {
      const int *p = I;
      I = J;
      J = p;
    }

  if (order == blas_colmajor)
    {
      if (beta == 0.0)
	ArraySetToZero(C, ldc, k, m);  /* C = 0.0 */
      else if (beta != 1.0)
	ArrayMultScalar(C, ldc, k, m, beta);  /* C = beta*C */

      if (alpha == 1.0)
	for (p=0; p<n; p++)
	  {
	    for (i=0; i<nnz; i++)
	      c[I[i]] += val[i] * b[J[i]];
	    c += ldc; b += ldb;
	  }
      else /* alpha != 1.0 */
	for (p=0; p<n; p++)
	  {
	    for (i=0; i<nnz; i++)
	      c[I[i]] += alpha * val[i] * b[J[i]];
	    c += ldc; b += ldb;
	  }
    }
  else if (order == blas_rowmajor)
    {
      if (beta == 0.0)
	ArraySetToZero(C, ldc, m, k);  /* C = 0.0 */
      else if (beta != 1.0)
	ArrayMultScalar(C, ldc, m, k, beta);  /* C = beta*C */

      if (alpha == 1.0)
	for (p=0; p<nnz; p++)
	  {
	    const double *bj = B + ldb*J[p];	/* Jth row of B */
	    double *ci = C + ldb*I[p];      	/* Ith row of C */
	    double v= val[p];
	    for (i=0; i<n; i++)
	      ci[i] += v * bj[i];
	  }
      else /* alpha != 1.0 */
	for (p=0; p<nnz; p++)
	  {
	    const double *bj = B + ldb*J[p];		/* Jth row of B */
	    double *ci = C + ldb*I[p];      		/* Ith row of C */
	    double v= val[p];
	    for (i=0; i<n; i++)
	      ci[i] += v * bj[i] * alpha;
	  }
    }
}

