/*  coomv.c  (Wed Aug  4 15:47:25 EDT 1999)
 *  Authors:  
 *  Galen Wilkerson and Roldan Pozo
 *  National Institute of Standards and Technology
 *
 *  Contact:  galen@cam.nist.gov or pozo@cam.nist.gov
 */

#include <stdio.h>
#include "spblas_i.h"
#include "c_blas.h"

void c_dusmv_COO( enum blas_trans_type T,  double alpha,
		  SPBLASI_double_COO A, double *x, int incx,  double beta, double *y, int incy )
{
  /********************************
    y <-  alpha * A * x +  beta * y
    ******************************/

  int i, j;
  double *p= y;
  
  double *pend;

  double *val = A->val_;
  int *I = A->i_;
  int *J = A->j_;
  int nz = A->nz_;
  int prop = A->prop_;
  
  if (T == blas_trans) /* swap I[] and J[] */
    {
      const int *p = I;
      I = J;
      J = p;
    }

  pend = y + A->M_*incy;

  if (incy == 1 && incx ==1 )
    {
      if (prop == sparse_blas_one_base) {
	y--;
	x--;
      }
      for (; p != pend;  *p++ *= beta);
	  
      for (i=0; i<nz; i++) { 
	y[ I[i] ] += alpha * x[ J[i] ] * val[i];
      }
     
    }
  else
    {
	  
      if (prop == sparse_blas_one_base) {
	y-=incy;
	x-=incx;
      }
	  	  
      for (; p<pend; p+=incy)
	*p *= beta;

      for (i=0; i<nz; i++)
	y[ I[i]*incy ] += alpha * x[ J[i]*incx ]  * val[i];
    }
}



/* symmetric matrix */
void c_dusmv_COOsymm( double alpha, SPBLASI_double_COO A, double *x, int incx,  double beta, double *y, int incy )
{

  /********************************
    y <-  alpha * A * x +  beta * y
    ******************************/

  int i, j;
  double *p= y;
  
  double *pend;

  double *val = A->val_;
  int *I = A->i_;
  int *J = A->j_;
  int nz = A->nz_;
  int prop = A->prop_;

  int rowi,coli;

  pend = y + A->M_*incy;

  if (incy == 1 && incx ==1 )
    {
      if (prop == sparse_blas_one_base) {
	y--;
	x--;
      }
      for (; p != pend;  *p++ *= beta);
	  
      for (i=0; i<nz; i++) { 
	y[ I[i] ] += alpha * x[ J[i] ] * val[i];
      }

      for (j=0;j<nz;j++) {
	rowi =  I[j];
	coli =  J[j];
	if ( rowi ==  coli )
	  y[rowi] += alpha * x[coli] * val[j];
	else {
	  y[rowi] += alpha * x[coli] * val[j];
	  y[coli] += alpha * x[rowi] * val[j];
	}
      }
     
    }
  else /* incx != 1 or incy != 1 */
    {
      if (prop == sparse_blas_one_base) {
	y-=incy;
	x-=incx;
      }
	  	  
      for (; p<pend; p+=incy)
	*p *= beta;

      for (j=0;j<nz;j++) {
	rowi =  I[j];
	coli =  J[j];
	if ( rowi ==  coli )
	  y[rowi * incy] += alpha * x[coli * incx] * val[j];
	else {
	  y[rowi * incy] += alpha * x[coli * incx] * val[j];
	  y[coli * incy] += alpha * x[rowi * incx] * val[j];
	}
      }
    }

}

