/*  coomm.c  (Wed Aug  4 15:47:25 EDT 1999)
 *  Authors:  
 *  Galen Wilkerson and Roldan Pozo
 *  National Institute of Standards and Technology
 *
 *  Contact:  galen@cam.nist.gov or pozo@cam.nist.gov
 */

#include <stdio.h>
#include "cblas.h"
#include "spblas_i.h"

void c_dusmm_COO( enum CBLAS_ORDER order, enum CBLAS_TRANSPOSE T,  int k, double alpha,
        SPBLASI_double_COO A, const double *B, int ldb, double beta, double *C, int ldc)
{
  /********************************

    C <- alpha * A * B + beta * C

    ******************************/

    int i, j;
    double *pC = C;

    double *pend;
        
    double *val = A->val_;
    int *I = A->i_;
    int *J = A->j_;
    int nz = A->nz_;
    int prop = A->prop_;
    int M = A->M_;
    int N = A->N_;
    
    if (prop == CBlasOneBase) {
      C--;
      B--;
    }
    
    if (T == CblasNoTrans) {
      
      if (order == CblasRowMajor) {	/* assumes both C and B are row-major */
      
	printf("case1: CblasNoTrans && CblasRowMajor ");

	/* premultiply C by beta */
	for(i=0;i < M; i++) {
	  pC = C + i * ldc;
	  pend = pC + k;
	  for(; pC < pend; *pC++ *= beta);
	}


      }  /*-------------end case 1-------------*/
      

      
      else {	/* assumes both C and B are column-major */

	printf("case2: CblasNoTrans && CblasColMajor ");

	/* premultiply C by beta */
	for (i = 0; i < k; i++) {
	  pC = C + (i * ldc);
	  pend = pC + M;
	  for(; pC < pend; *pC++ *= beta);
	}
      }

      
      /* loop through the rows of C one at a time */
      for(i = 0;i != N; i++) {
	/* loop through the nonzero entries of A */
	for(j = 0;j != nz; j++) {
	  C[J[j]] += alpha * B[I[j]] * val[j]; 
	}
	C += ldc; B += ldb;
      }  /*-------------end case 2-------------*/


    }
    else {
      
      if (order == CblasRowMajor) { 	/* assumes both C and B are row-major */
	
	printf("case3: CblasTrans && CblasRowMajor ");

	/* premultiply C by beta */
	for(i=0;i < M; i++) {
	  pC = C + i * ldc;
	  pend = pC + k;
	  for(; pC < pend; *pC++ *= beta);
	}
      } /*-------------end case 3-------------*/
      
            
      else{ /* assumes both C and B are column-major */
	
	printf("case4: CblasTrans && CblasColMajor ");

	/* premultiply C by beta */
	for (i = 0; i < k; i++) {
	  pC = C + (i * ldc);
	  pend = pC + M;
	  for(; pC < pend; *pC++ *= beta);
	}
      }


      /* loop through the columns of C one at a time */
      for(i = 0;i != N; i++) {
	/* loop through the nonzero entries of A */
	for(j = 0;j != nz; j++) {
	  C[I[j]] += alpha * B[J[j]] * val[j]; 
	}
	C += ldc; B += ldb;
      }
    }
}
