#include <stdio.h>
#include <stdlib.h>
#include "c_blas.h"

void dgemm_col_notrans(int M, int N, int K, double alpha, 
	const double *A, int ldA, const double *B, int ldB,
	double beta, double *C, int ldC);

void dgemm_row_notrans(int M, int N, int K, double alpha, 
	const double *A, int ldA, const double *B, int ldB,
	double beta, double *C, int ldC);

void c_dgemm(enum blas_order_type order, enum blas_trans_type transA, 
	enum blas_trans_type transB, int M, int N, int K, double alpha,
	const double *A, int ldA, const double *B, int ldB, double beta,
	double *C, int ldC)
{
	if (transA == blas_trans || transB == blas_trans)	
	{
		printf("dgemm: transpose not supported.\n");
		exit(1);
	}

	if (order == blas_rowmajor)
	{
		dgemm_row_notrans(M, N, K, alpha, A, ldA, B, ldB, beta, C, ldC);
	}
	else if (order == blas_colmajor)
	{
		if (transA == blas_no_trans && transB == blas_no_trans)
			dgemm_col_notrans(M, N, K, alpha, A, ldA, B, ldB, beta, C, ldC);
	
	}
}


void dgemm_col_notrans(int M, int N, int K, double alpha, 
	const double *A, int ldA, const double *B, int ldB,
	double beta, double *C, int ldC)
{
	int i, j, k, kldA;

	for (i=0; i<M; i++)
	{
		const double *Arowi = A + i;
		double *Crowi = C + i;

		for (j=0; j<K; j++)
		{
			double t = 0.0;
			const double *Bcolj = B + j*ldB;

			for (k=0, kldA=0; k<N; k++, kldA += ldA)
				t += Arowi[kldA] * Bcolj[k];
			Crowi[j*ldC] = alpha * t + beta * Crowi[j*ldC];
		}
	}
}

void dgemm_row_notrans(int M, int N, int K, double alpha, 
	const double *A, int ldA, const double *B, int ldB,
	double beta, double *C, int ldC)
{
	int i, j, k, kldB;

	for (i=0; i<M; i++)
	{
		const double *Arowi = A + ldA*i;
		double *Crowi = C + ldC*i;

		for (j=0; j<K; j++)
		{
			double t = 0.0;
			const double *Bcolj = B + j;

			for (k=0, kldB=0; k<N; k++, kldB += ldB)
				t += Arowi[k] * Bcolj[kldB];
			Crowi[j] = alpha * t + beta * Crowi[j];
		}
	}
}

