#include <malloc.h>
#include "spblas_i.h"
#include "c_blas.h"

/* create a new (0-based) compressed-column sparse matrix from a 
		coordinate sparse matirx */

DCSC DCOO_2_DCSC(DCOO A)
{

	int i=0;
	int M = A->M_;
	int N = A->N_;
	int *I = A->i_;
	int *J = A->j_;
	double *val = A->val_;
	int nz = A->nz_;
	int properties = A->prop_;
	int index = 0;


	int index_base = A->prop_ || sparse_blas_zero_base;

	/* components for CSC routine */

		
	double *CSC_val = NULL;
	int *CSC_pntrb = NULL;
	int *CSC_j = NULL;
	int *tally = NULL;


	CSC_val = (double *) malloc(nz * sizeof(double));
	CSC_pntrb = (int *) malloc( (N+1) * sizeof (int));
	CSC_j = (int *) malloc(nz*sizeof(int));
	tally = (int *) malloc( (N) * sizeof (int));

	/* initialize count to zero */
	for (i=0; i<N; i++)
	  tally[i] = 0;

	/* 1st pass: find out how many elements in each column */

	for (i=0; i<nz; i++)
	  tally[J[i] - index_base]++;


	/* convert tally[] into (0-based) row pointer by accumulating
		row-lengths.  tally[i] now points to the begining or row i  */

	for (i=1;i<N; i++)
		tally[i] += tally[i-1];


	/* copy these values into pntrb[]; (we will be using the values
		of tally[] as place-markers, which change as we fill up the
		CSC matrix). */

	CSC_pntrb[0]=index_base;
	for (i=0; i<N; i++)
		CSC_pntrb[i+1] = tally[i] + index_base;

	/* copy back original row-starts into tally */
	for (i=0; i<N; i++)
		tally[i] = CSC_pntrb[i] - index_base;

	/* 2nd pass: now go through and fill-in CSC format */

	for (i=0; i<nz; i++)
	{
		index = tally[J[i] - index_base]++;
		CSC_val[index] = val[i];
		CSC_j[index] = I[i];
	}

	free(tally);
	return create_DCSC(M, N, nz, CSC_val, CSC_j, CSC_pntrb, CSC_pntrb+1, 
		properties);

}
		



