/**
  
  @author Galen J. Wilkerson
  
  */
import java.io.*;

public class TestMult {
  
  public TestMult() {

  }
  
  
  
  /** 
    test driver
    */
  public static void main(String args[]) {
    
    double alpha, beta;
    int M;
    int N;
    int nz;
    int k;

    int transpose = Jblas.NoTrans;
    
    // the compressed row matrix for testing
    DCSR CSRA;

    
    // initialize matrix A - used for reading matrix market format
    DCOO A = new DCOO();
    
    try {
      A.read("Data/pores_1.mtx");
    }
    catch (IOException e) {
      e.printStackTrace();
    }
     
    M = A.getNumRows();
    N = A.getNumCols();
    nz = A.getNumNonZero();


    // convert the DCOO object A into the DCSR object CSRA
    CSRA = new DCSR(A);

    System.out.println("DCOO A:");
    
    A.print();

    System.out.println("\nDCSR CSRA:");

    CSRA.print2();

    // initialize matrix B
    double B[][];
    try {
      B = Dense.read("Data/B30rowmaj");
    }
    catch (IOException e) {
      e.printStackTrace();
      return;
    }
     
    k = B[0].length;
     

    double C[][] = new double[M][k];

    // initialize matrix BB2 (for submatrix case - CASE 13)
    double BB2[][];

    try {
      BB2 = Dense.read("Data/BB60rowmaj");
    }
    catch (IOException e) {
      e.printStackTrace();
      return;
    }
    
    int BB2k = BB2[0].length;    

    double CC[][] = new double[2*M][BB2k];


    // CASE 0 -      alpha == 1.0 && beta == 0.0
    alpha = 1.0;
    beta = 0.0;

    // zero-out C
    Dense.arraySetToValue(C,0.0);

    //    A.dusmm(transpose, k, alpha, B, beta, C);
    CSRA.dusmm(transpose, k, alpha, B, beta, C);

    try {
      Dense.save(C, "Data/outC0");
    }
    catch(IOException e) {
      e.printStackTrace();
    }

    


    // CASE 2 -      alpha == 0.0 && beta != 0.0
    alpha = 0.0;
    beta = -3.75;
    

    // init C to all 1.0
    Dense.arraySetToValue(C,1.0);

    //    A.dusmm(transpose, k, alpha, B, beta, C);
    CSRA.dusmm(transpose, k, alpha, B, beta, C);

    try {
      Dense.save(C, "Data/outC2");
    }
    catch(IOException e) {
      e.printStackTrace();
    }



    // CASE 3 -      alpha == 1.0 && beta == 0.0 && transpose
    transpose = Jblas.Trans;
    alpha = 1.0;
    beta = 0.0;

    // zero-out C
    Dense.arraySetToValue(C,0.0);
    
    //    A.dusmm(transpose, k, alpha, B, beta, C);
    CSRA.dusmm(transpose, k, alpha, B, beta, C);

    try {
      Dense.save(C, "Data/outC3");
    }
    catch(IOException e) {
      e.printStackTrace();
    }

    transpose = Jblas.NoTrans;




    // CASE 8 & 10 - beta == 0.0 && alpha == 1.0
    alpha = 1.0;
    beta = 0.0;

    Dense.arraySetToValue(C,0.0);
    
    double[][] B2 = Dense.getTranspose(B);

    //A.dusmm(transpose, k, alpha, B2, beta, C);
    CSRA.dusmm(transpose, k, alpha, B2, beta, C);

    try {
      Dense.save(C, "Data/outC8");
    }
    catch(IOException e) {
      e.printStackTrace();
    }

    
    // CASE 9 & 11 - beta != 0.0 && alpha != 1.0
    alpha = 2.25;
    beta = -3.75;
    
    Dense.arraySetToValue(C,1.0);
    
    //A.dusmm(transpose, k, alpha, B2, beta, C);
    CSRA.dusmm(transpose, k, alpha, B2, beta, C);

    try {
      Dense.save(C, "Data/outC9");
    }
    catch(IOException e) {
      e.printStackTrace();
    }


    // CASE 13 -     B, C submatrices
    alpha = 1.0;
    beta = 0.0;
    
    Dense.arraySetToValue(CC,0.0);
    
    //    Dense.print(BB2);
    
    //    A.dusmm(transpose, k, alpha, BB2, N/2, k/2, beta, CC, N, k);
    CSRA.dusmm(transpose, k, alpha, BB2, N/2, k/2, beta, CC, N, k);

    try {
      Dense.save(CC, 30, 30, M, k, "Data/outC13");
    }
    catch(IOException e) {
      e.printStackTrace();
    }
  }
}
