import java.util.*;
import java.io.*;

public class CompRow {

  // the actual elements
  double[] val;
 
  // the column indices 
  int[] indx;

  // the row beginning points
  int[] pntrb;

  // the row end points
  int[] pntre;

  // dimensions of the array
  int height;
  int width;

  int numNonZero;

  // init the compressed array
  public CompRow() {
  }


  public CompRow(double[] valArray, int[] indxArray, int[] pntrbArray, 
		 int[] pntreArray, int tempHeight, int tempWidth, int tempNumNonZero) 
  {
    val = valArray;
    indx = indxArray;
    pntrb = pntrbArray;
    pntre = pntreArray;
    height = tempHeight;
    width = tempWidth;
    numNonZero = tempNumNonZero;
  }


  public void print() {
    
    System.out.println(" Val:  ");
    for (int i = 0; i < numNonZero; i++) {
      System.out.print(val[i]+ " ");
    }
    System.out.println();

    System.out.println(" indx:  ");
    for (int i = 0; i < indx.length; i++) {
      System.out.print(indx[i] + " ");
    }
    System.out.println();
    
    System.out.println(" pntrb:  ");
    for (int i = 0; i < pntrb.length; i++) {
      System.out.print(pntrb[i] + " ");
    }
    System.out.println();

    System.out.println(" pntre:  ");
    for (int i = 0; i < pntre.length; i++) {
      System.out.print(pntre[i]+ " ");
    }
    System.out.println();
  }


  public static CompRow dense2Sparse(double[][] inArray) {

    Vector tempVal = new Vector();
    Vector tempIndx = new Vector();
    Vector tempPntrb = new Vector();
    int tempNumNonZero = 0;

    double[]valArray;
    int[] indxArray;
    int[] pntrbArray;
    int[] pntreArray;
    
    int tempHeight = inArray.length;
    int tempWidth = inArray[0].length;
    
    // column loop
    for (int i = 0; i < inArray.length; i++) {
      tempPntrb.addElement(new Integer(tempNumNonZero));
      
      // row loop
      for (int j = 0; j < inArray[0].length; j++) {
	if (inArray[i][j] != 0) {
	  tempVal.addElement(new Double(inArray[i][j]));
	  tempIndx.addElement(new Integer(j));
	  tempNumNonZero++;
	}
      }      
    }

    valArray = new double[tempNumNonZero];
    indxArray = new int[tempNumNonZero];
    pntrbArray = new int[tempPntrb.size()];
    pntreArray = new int[tempPntrb.size()];

    for (int i = 0; i < tempVal.size() ; i++) {
      valArray[i] = ((Double) tempVal.elementAt(i)).doubleValue();
    }
    
    for (int i = 0; i < tempIndx.size() ; i++) {
      indxArray[i] = ((Integer) tempIndx.elementAt(i)).intValue();
    }
    
    for (int i = 0; i < tempPntrb.size() ; i++) {
      pntrbArray[i] = ((Integer) tempPntrb.elementAt(i)).intValue();
    }

    for (int i = 0; i < (tempPntrb.size() - 1) ; i++) {
      pntreArray[i] = pntrbArray[i+1];
    }
    pntreArray[tempPntrb.size() - 1] = tempNumNonZero;
    
    return (new CompRow(valArray, indxArray, pntrbArray, pntreArray, tempHeight, tempWidth,tempNumNonZero));
  }
  

  /*
  // multiply by a vector
  public double[] standardMult(double[] inVector) {
    double[] resultVector = new double[height];
    int temp;

    // standard matrix multiply     
    for (int i = 0; i < height; i++) {
      temp = 0;
      for (int j = 0; j < inVector.length; j++) {
	temp += inVector[j] * sourceArray[i][j];
      }
      resultVector[i] = temp;
    }
    return resultVector;
  }
  */


  public double[] mult(double[] inVector) {
    
    double[] resultVector = new double[height];
    int temp;
    
    for (int rowNum = 0; rowNum < height; rowNum++) {
      for (int xindex = pntrb[rowNum]; xindex < pntre[rowNum]; xindex++) {
	// multiply the inVector by the array's row
	resultVector[rowNum] += (inVector[indx[xindex]] * val[xindex]);
      }
    }
    return resultVector;
  }



  public static void main(String[] args) {

    //    double[][] tempArray = {{11,0,13,14,0},{0,0,23,24,0},{31,32,33,34,0},{0,42,0,44,0},{51,52,0,0,55}};
    double[] tempVect = {1,1,1,1,1};
    CompRow c1;

    try {
      
      NumericReader nr = new NumericReader( new FileReader("dense.dat"));
      int h1 = nr.readInt();
      int w1 = nr.readInt();
      
      // read the dense array
      double[][] tempArray = new double[h1][w1];
      
      for (int i = 0; i < h1 * w1; i++) {
	tempArray[(i / w1)] [(i % w1)] = nr.readDouble();
      }
      
      c1 = CompRow.dense2Sparse(tempArray);
      c1.print();
      System.out.println();
      
      // do a compressed multiplication
      double[] resultVect = c1.mult(tempVect);
      for (int i = 0; i < resultVect.length; i++) {
	System.out.println(resultVect[i]);
      }
    }
    
    catch(Exception e) {
      e.printStackTrace();
    }
  }
}




