/**
  
  @author Galen J. Wilkerson
  
  */
import java.io.*;

public class TestMultLU {
  
  public TestMultLU() {

  }
  
  
  
  /** 
    test driver
    */
  public static void main(String args[]) {
    
    double alpha, beta;
    int M;
    int N;
    int nz;
    int k;

    int transpose = Jblas.NoTrans;
    
    // the compressed row matrix for testing
    DCSR CSRA, CSRA_1;

    
    // initialize matrix A - used for reading matrix market format
    DCOO A = new DCOO();
    
    try {
      A.read("LU_Data/pores_1_LU.mtx");
    }
    catch (IOException e) {
      e.printStackTrace();
    }
     
    M = A.getNumRows();
    N = A.getNumCols();
    nz = A.getNumNonZero();


    // convert the DCOO object A into the DCSR object CSRA
    CSRA = new DCSR(A);

    System.out.println("DCOO A:");
    
    A.print();

    System.out.println("\nDCSR CSRA:");

    CSRA.print2();



    // read the DCOO object having a unitary diagonal
    // for verification using matrix multiplication
    A = new DCOO();
    
    try {
      A.read("LU_Data/pores_1_LU1.mtx");
    }
    catch (IOException e) {
      e.printStackTrace();
    }
     
    // create the DCSR object 
    CSRA_1 = new DCSR(A);

    System.out.println("\nDCSR CSRA_1:");
    CSRA_1.print2();



    // initialize matrix B
    double B[][];
    try {
      B = Dense.read("LU_Data/B30rowmaj");
    }
    catch (IOException e) {
      e.printStackTrace();
      return;
    }
     
    k = B[0].length;

    double C[][] = new double[M][k];

    double B0[][], B3[][], B4[][];



    double BB[][];
    int BBi = 0, BBj = 0;


    try {
      BB = Dense.read("LU_Data/BB60rowmaj");
    }
    catch (IOException e) {
      e.printStackTrace();
      return;
    }

    double CC[][] = new double[2*M][2*k];
    int CCi, CCj;


    // CASE 0 -      alpha == 1.0 , no transpose
    alpha = 1.0;
    beta = 0.0;

    // zero-out C
    Dense.arraySetToValue(C,0.0);

    // copy B to B0
    B0 = new double[N][k];
    for (int i = 0; i < N; i++)
      System.arraycopy(B[i], 0, B0[i], 0, k);
    
    //    System.out.println("original B");
    //Dense.print(B);

    // triangular solve
    CSRA.dussmLU(transpose, alpha, k, B, 0, 0);
    
    //    System.out.println("result B");
    //Dense.print(B);

    try {
      Dense.save(B, "LU_Data/outB0");
    }
    catch(IOException e) {
      e.printStackTrace();
    }

    // verify results using matrix multiply
    CSRA_1.dusmm(transpose, k, alpha, B, beta, C);

    //    System.out.println("C (should equal original B)");
    //Dense.print(C);

    // print the norm of C - B0
    System.out.println("Norm(C - B0)\n"+Dense.matrixNorm(C, B0,  M,  k));






    // CASE 3 -      alpha == 1.0 && beta == 0.0 && transpose
    transpose = Jblas.Trans;
    alpha = 1.0;
    beta = 0.0;

    // zero-out C
    Dense.arraySetToValue(C,0.0);


    // re-read B
    try {
      B = Dense.read("LU_Data/B30rowmaj");
    }
    catch (IOException e) {
      e.printStackTrace();
      return;
    }


    B3 = new double[N][k];
    for (int i = 0; i < N; i++)
      System.arraycopy(B[i], 0, B3[i], 0, k);

    System.out.println("original B");
    Dense.print(B);
    
    // triangular solve
    CSRA.dussmLU(transpose, alpha, k, B, 0, 0);

    System.out.println("result B");
    Dense.print(B);

    try {
      Dense.save(B, "LU_Data/outB3");
    }
    catch(IOException e) {
      e.printStackTrace();
    }

    // verify results using matrix multiply
    CSRA_1.dusmm(transpose, k, alpha, B, beta, C);

    System.out.println("C (should equal original B)");
    Dense.print(C);

    // print the norm of C - B3
    System.out.println("Norm (C - B3)\n" + Dense.matrixNorm(C, B3,  M,  k));



    //  CASE 4 - submatrix 
    //  no transpose, alpha == 3.25
    transpose = Jblas.NoTrans;
    alpha = 3.25;
    beta = 0.0;

    Dense.arraySetToValue(CC,0.0);
    
    B4 = new double[2 * N][2 * k];
    for (int i = 0; i < 2* N; i++)
      System.arraycopy(BB[i], 0, B4[i], 0, 2 * k);

    System.out.println("original BB");
    Dense.print(BB);

    // triangular solve
    CSRA.dussmLU(transpose, alpha, k, BB, M/2, N/2);

    //System.out.println("result BB");
    //Dense.print(BB);

    try {
      Dense.save(BB, "LU_Data/outB4");
    }
    catch(IOException e) {
      e.printStackTrace();
    }

    // verify the results using matrix multiply 
    CSRA_1.dusmm(transpose, k, 1/alpha, BB, M/2, k/2, beta, CC, M/2, N/2);

    System.out.println("CC (should equal original BB)");
    Dense.print(CC);

    // print the norm of CC-B4
    System.out.println("Norm(CC - B4)\n"+Dense.matrixNorm(CC, M/2, N/2, B4, M/2, N/2, M, k));

  }
}
