/**
  
  @author Galen J. Wilkerson
  
  */
import java.io.*;

public class TestMultLU {
  
  public TestMultLU() {

  }
  
  
  
  /** 
    test driver
    */
  public static void main(String args[]) {
    
    double alpha, beta = 0;
    int M;
    int N;
    int nz;

    int transpose = Jblas.NoTrans;
    
    // the compressed row matrix for testing
    DCSR CSRA, CSRA_1;

    // initialize matrix A
    DCOO A = new DCOO();
    
    try {
      A.read("LU_Data/pores_1_LU.mtx");
    }
    catch (IOException e) {
      e.printStackTrace();
    }
     
    M = A.getNumRows();
    N = A.getNumCols();
    nz = A.getNumNonZero();


    System.out.println("DCOO A:");
    
    A.print();
    

    // convert the DCOO object A into the DCSR object CSRA
    CSRA = new DCSR(A);

    System.out.println("\nDCSR CSRA:");
    CSRA.print2();


    // read the DCOO object having a unitary diagonal
    // for verification using matrix multiplication
    A = new DCOO();
    
    try {
      A.read("LU_Data/pores_1_LU1.mtx");
    }
    catch (IOException e) {
      e.printStackTrace();
    }
     
    // create the DCSR object 
    CSRA_1 = new DCSR(A);

    System.out.println("\nDCSR CSRA_1:");
    CSRA_1.print2();



    // initialize vector x0
    double x0[] = new double[N];
    int incx0 = 1;

    double y0[] = new double[N];
    int incy0 = 1;

    // initialize vector x1
    double x1[];
    int incx1 = 1;

    try {
      x1 = Dense.readVector("LU_Data/x1");
    }
    catch (IOException e) {
      e.printStackTrace();
      return;
    }

    double y1[] = new double[M];
    int incy1 = 1;

     

    // initialize vector x6
    double x6[];
    int incx6 = 2;

    try {
      x6 = Dense.readVector("LU_Data/x6");
    }
    catch (IOException e) {
      e.printStackTrace();
      return;
    }
    
    double y6[] = new double[M];
    int incy6 = 1;
    

    /****************************
      Case 0
      transpose = no trans
      incx = 1
      incy = 1
      
      ****************************/


    alpha = 1.0;
    beta = 0.0;

    Dense.arraySetToValue(x0,incx0,1.0);

    // setup a vector to verify results
    double tempVect[] = new double[x0.length];
    int tempInc = incx0;
    System.arraycopy(x0, 0, tempVect, 0, x0.length);

    System.out.println("\nx0");
    Dense.printVector(x0);

    // triangular solve
    CSRA.dussvLU(transpose, alpha, x0, incx0);

    
    System.out.println("\nx0");
    Dense.printVector(x0);

    // verify with vector multiplication
    CSRA_1.dusmv(transpose, alpha, x0, incx0, beta, y0, incy0);

    System.out.println("\ntempVect");
    Dense.printVector(tempVect);

    System.out.println("\ny0");
    Dense.printVector(y0);


    // print the norm of tempVect and y0
    System.out.println("Norm(origX0 - y0:\n" + Dense.vectorNorm(tempVect, y0, y0.length));

    try {
      Dense.save(x0, incx0,"LU_Data/outX0");
    }
    catch(IOException e) {
      e.printStackTrace();
    }

    
    /****************************
      Case 1
      transpose = trans
       
      ****************************/

    // clear tempVect
    Dense.arraySetToValue(tempVect,tempInc,0);
    System.arraycopy(x1, 0, tempVect, 0, x1.length);

    

    transpose = Jblas.Trans;
    CSRA.dussvLU(transpose, alpha, x1, incx1);
    
    // verify with vector multiplication
    CSRA_1.dusmv(transpose, alpha, x1, incx1, beta, y1, incy1);
    
    // print the norm of tempVect and y1
    System.out.println("Norm(origX1 - y1:\n" + Dense.vectorNorm(tempVect, y1, y1.length));


    try {
      Dense.save(x1, incx1,"LU_Data/outX1");
    }
    catch(IOException e) {
      e.printStackTrace();
    }


    /****************************
      Case 6
      incx6 = 2
      
      ****************************/

    // setup a vector to verify results
    tempVect = new double[x6.length];
    tempInc = incx6;
    System.arraycopy(x6, 0, tempVect, 0, x6.length);
   
    alpha = -3.75;

    // triangular solve
    transpose = Jblas.NoTrans;
    CSRA.dussvLU(transpose, alpha, x6, incx6);

    // verify with vector multiplication
    CSRA_1.dusmv(transpose, 1/alpha, x6, incx6, beta, y6, incy6);

    // print the norm of tempVect and y6
    System.out.println("Norm(origX6 - y6:\n" + Dense.vectorNorm(tempVect, tempInc, y6, incy6, y6.length));

    try {
      Dense.save(x6, incx6, "LU_Data/outX6");
    }
    catch(IOException e) {
      e.printStackTrace();
    }
  }
}

