/**

  @author Galen J. Wilkerson

 */

public class DCOO {
  
  int M_, N_, nz_;
  int prop_;
  double[] val_;          /* length = nz */
  int[] indx_;             
  int[] jndx_;             


  public DCOO() {
  }


  /** 
    constructor
   */
  public DCOO( int M, int N, int nz, double[] val, int[] indx, int[] jndx, int properties) 
  {
    M_ = M;
    N_ = N;
    nz_ = nz;
    val_ = new double[val.length];
    indx_ = new int[indx.length];
    jndx_ = new int[jndx.length];
    
    for (int i = 0; i < val.length; i++) {
      val_[i] = val[i];
      indx_[i] = indx[i];
      jndx_[i] = jndx[i];
    }

    prop_ = properties;
  }

  /**
    print the current object
   */
  public void print() {
    System.out.println("M " + M_);
    System.out.println("N " + N_);
    System.out.println("nz " + nz_);

    System.out.println("val   indx    jndx");
    for (int i = 0; i < val_.length; i++) {
      System.out.println(val_[i] + "       " + indx_[i] + "       " + jndx_[i]);
    }

    System.out.println("properties " + prop_);
  }


  /**
    Read the file and return a dense matrix
   */
  public static double[][] readDense(String fileName) {
    try {
      NumericReader nr = new NumericReader(new FileReader(fileName));
      int h1 = nr.readInt();
      int w1 = nr.readInt();
      
      // read the dense array
      double[][] tempArray = new double[h1][w1];
      
      for (int i = 0; i < h1 * w1; i++) {
	tempArray[(i / w1)] [(i % w1)] = nr.readDouble();
      }
    }
    catch (Exception e) {
      e.printStackTrace();
    }
  }


  /** 
    Read into the current object from matrix-market format.
   */
  public void readSparse() {
    
  }


  /**
    static method to initialize a dense matrix
   */
  public static void arraySetToValue(double[][] matrix, double value) {
    for(int i = 0; i < matrix.length; i++) 
      for(int j = 0; j < matrix[0].length; j++)
	matrix[i][j] = value;
    return;
  }



  /**
    static method to scale a dense matrix
   */
  public static void arrayMultScalar(double[][] matrix, double scalar) {
    for(int i = 0; i < matrix.length; i++) 
      for(int j = 0; j < matrix[0].length; j++) 
	matrix[i][j] *= scalar;
    return;
  }
  

  /**
    method to support CaABbC matrix-matrix multiplication
   */
  public void j_dusmm_COO(final int transpose, int k, double alpha,
			  double B[][], double beta, double C[][]) 
  { 
    if (alpha == 0) 
      {
	if (beta != 1.0) {
	  arrayMultScalar(C, beta); 
	  return;
	}
      }
    
    if (transpose == Jblas.Trans) { // swap indx_ and jndx_ 
      int[] temp = indx_;
      indx_ = jndx_;
      jndx_ = temp;
    }

    if (beta == 0.0) 
      arraySetToValue(C,0.0);
    else if (beta != 1.0) 
      arrayMultScalar(C,beta);

    if (alpha == 1.0) {
      for (int i = 0; i < k; i++) 
	for (int j = 0; j < nz_; j++) {
	  C[indx_[j]][i] += val_[j] * B[jndx_[j]][i];
	}
    }
    else {  // alpha != 1.0
      for (int i = 0; i < k; i++) 
	for (int j = 0; j < nz_; j++) {
	  C[indx_[j]][i] += val_[j] * B[jndx_[j]][i] * alpha;
	}
    }
  }

  /** 
    test driver - more detailed testing done by TestMult.java
   */
  public static void main(String args[]) {
    int M = 5;
    int N = 5;
    int nz = 5;
    double val[] = {1.0,1.0,1.0,1.0,1.0};
    int indx[] = {0,1,2,3,4};
    int jndx[] = {0,1,2,3,4};
    int properties = 0;

    int transpose = Jblas.NoTrans;
    int k = 5;
    double alpha = 2.25;
    double beta = -3.75;

    DCOO dc1 = new DCOO(M, N, nz, val, indx, jndx, properties); 

    dc1.print();

    double[][] B = {{2.0,2.0,2.0,2.0,2.0},
		    {2.0,2.0,2.0,2.0,2.0},
		    {2.0,2.0,2.0,2.0,2.0},
		    {2.0,2.0,2.0,2.0,2.0},
		    {2.0,2.0,2.0,2.0,2.0}};


    double[][] C = new double[5][5];
    DCOO.arraySetToValue(C,1.0);
        
    dc1.j_dusmm_COO(transpose, k, alpha, B, beta, C);
  
    for (int i = 0; i < C.length; i++) {
      for (int j = 0; j < C[0].length; j++) 
	System.out.print(C[i][j]+ "  ");
      System.out.println();
    }
  }
}

