import java.util.*;
import java.io.*;
//import jnt.io.*;

public class Coordinate {

  double[] val;
  int[] iindex;
  int[] jindex;

  int height;
  int width;
  int numNonZero;

  public Coordinate() {
  }

  public Coordinate(double[] inVal, int[] inIindex,int[] inJindex, int inHeight,int inWidth,int inNumNonZero) 
  {
    val = inVal;
    iindex = inIindex;
    jindex = inJindex;
    height = inHeight;
    width = inWidth;
    numNonZero = inNumNonZero;
  }

  public static Coordinate dense2Sparse(double[][] inArray) {

    Vector tempVal = new Vector();
    Vector tempIindex = new Vector();
    Vector tempJindex = new Vector();
    
    double[] valArray;
    int[] iindexArray;
    int[] jindexArray;

    int tempHeight = inArray.length;
    int tempWidth = inArray[0].length;
    
    for (int i = 0; i < tempHeight; i++) {
      for (int j = 0; j < tempWidth; j++) {
	if (inArray[i][j] != 0) { 
	  tempVal.addElement(new Double(inArray[i][j]));
	  tempIindex.addElement(new Integer(i));
	  tempJindex.addElement(new Integer(j));
	}
      }
    }
    
    int tempNumNonZero = tempVal.size();
    valArray = new double[tempNumNonZero];
    iindexArray = new int[tempNumNonZero];
    jindexArray = new int[tempNumNonZero];
    
    for (int i = 0; i < tempNumNonZero ; i++) {
      valArray[i] = ((Double) tempVal.elementAt(i)).doubleValue();
      iindexArray[i] = ((Integer) tempIindex.elementAt(i)).intValue();
      jindexArray[i] = ((Integer) tempJindex.elementAt(i)).intValue();
    }
    
    return (new Coordinate(valArray,iindexArray,jindexArray,tempHeight,tempWidth,tempNumNonZero));
  }


  public double[] mult(double[] inVector) {
    double[] resultVector = new double[height];

    for (int i = 0; i < numNonZero; i++) {
      resultVector[iindex[i]] += val[i] * inVector[jindex[i]];
    }
    
    return resultVector;
  }


  public void print() {

    System.out.println(height + " " + width + " " + numNonZero);

    for (int i = 0; i < val.length; i++) {
      System.out.println(iindex[i]+ " " + jindex[i]+ " " + val[i]);
    }
  }


  public static void main(String[] args) {
    
    //double[][] tempArray = {{11,0,13,14,0},{0,0,23,24,0},{31,32,33,34,0},{0,42,0,44,0},{51,52,0,0,55}};
    double[] tempVect = {1,1,1,1,1};
    double[] outVect;
    Coordinate c1;

    try {

      NumericReader nr = new NumericReader( new FileReader("dense.dat"));
      int h1 = nr.readInt();
      int w1 = nr.readInt();

      // read the dense array
      double[][] tempArray = new double[h1][w1];

      for (int i = 0; i < h1 * w1; i++) {
	tempArray[(i / w1)] [(i % w1)] = nr.readDouble();
      }

      // print it
      for (int i = 0; i < tempArray.length; i++) {
	for (int j = 0; j < tempArray[0].length; j++) {
	  System.out.print(tempArray[i][j]+"  ");
	}
	System.out.println();
      }
      System.out.println();

      // convert the dense array to a sparse coordinate array
      c1 = Coordinate.dense2Sparse(tempArray);
      c1.print();

      System.out.println();

      // multiply the compressed coordinate array by the input
      outVect = c1.mult(tempVect);
    
      for (int i = 0; i < outVect.length; i++) {
	System.out.print(outVect[i]+ "  ");
      }
      System.out.println();
    }
  
    catch(Exception e) {
      e.printStackTrace();
    }
  }
}




