import java.io.*;
import java.util.*;

public class Dense {

  /**
    static method to initialize a dense matrix.
   */
  public static void arraySetToValue(double[][] matrix, final double value) {
    for(int i = 0; i < matrix.length; i++) 
      for(int j = 0; j < matrix[0].length; j++)
	matrix[i][j] = value;
    return;
  }


  /**
    static method to scale a dense matrix.
   */
  public static void arrayMultScalar(double[][] matrix, final double scalar) {
    for(int i = 0; i < matrix.length; i++) 
      for(int j = 0; j < matrix[0].length; j++) 
	matrix[i][j] *= scalar;
    return;
  }

  /**
    static method to initialize a dense vector
   */
  public static void arraySetToValue(double[] vector, final int incv, final double value) {
    for(int i = 0; i < vector.length; i++) 
      vector[i * incv] = value;
    return;
  }


  /**
    static method to scale a dense vector.
   */
  public static void arrayMultScalar(double[] vector, final int incv, final double scalar) {
    for(int i = 0; i < vector.length; i++) 
      vector[i * incv] *= scalar;
    return;
  }
  

  /**
    Return the transpose of a matrix.
   */
  public static double[][] getTranspose(double[][] matrix) {
    double[][] tempMatrix = new double[matrix[0].length][matrix.length];
    
    for(int i = 0; i < matrix.length; i++)
      for(int j = 0; j < matrix.length; j++)
	tempMatrix[j][i] = matrix[i][j];

    return tempMatrix;
  }

  /**
    create a dense identity matrix of double values

    NEED TO THROW EXCEPTION
    */
  public static double[][] createIdentity(int M, int N) {
    double[][] tempMatrix = new double [M][N];

    // Note: Java initializes double[][] arrays to 0.0
    if (M == N) 
      for (int i = 0; i <  M; i++)
	tempMatrix[i][i] = 1.0;
    else {
      System.err.println("Error: M != N, could not create matrix.");
    }
    return tempMatrix;
  }
  

  /**
    Return a dense submatrix.
    */
  public static void getSubMatrix(final double[][] matrix, final int rowOffset, final int colOffset,
				  final int M, final int N) throws IOException 
  {
    double[][] tempMatrix = new double[M][N];
    
    for (int i = 0; i <  M; i++)
      for (int j = 0; j < N; j++)
	tempMatrix[i][j] = matrix[i + rowOffset][j + colOffset];
  }


  /**
    Read the file and return a dense matrix.
   */
  public static double[][] read(final String fileName) throws IOException {
    NumericReader nr = new NumericReader(new FileReader(fileName));
    int h1 = nr.readInt();
    int w1 = nr.readInt();
    
    // read the dense array
    double[][] tempArray = new double[h1][w1];
    
    for (int i = 0; i < h1 * w1; i++) {
      tempArray[(i / w1)] [(i % w1)] = nr.readDouble();
    }
    
    return tempArray;
  }


  /**
    Read the file and return a dense vector.
   */
  public static double[] readVector(final String fileName) throws IOException {
    NumericReader nr = new NumericReader(new FileReader(fileName));
    int h1 = nr.readInt();
    
    // read the dense array
    double[] tempArray = new double[h1];
    
    for (int i = 0; i < h1; i++) {
      tempArray[i] = nr.readDouble();
    }
    
    return tempArray;
  }

  /**
    Find the Norm of two vectors
    */
  public static double vectorNorm(double[] X1, double[] X2, int height) {
    return vectorNorm(X1, 1, X2, 1, height);
  }

  /**
    Find the Norm of two vectors - allows increments > 1
    */
  public static double vectorNorm(double[] X1, int incx1, double[] X2, int incx2, int height) {
    
    double sum = 0;
    int i;
    for (i = 0; i < height; i++) {
      sum += Math.pow(X1[i * incx1] - X2[i * incx2], 2);
    }
    
    return(Math.sqrt(sum)/(double)height);
  }


  /** 
    Find the Norm of two matrices 
   */
  public static double matrixNorm(double[][] A, double[][] B, int numRows, int numCols){
    
      return(matrixNorm(A,0,0,B,0,0,numRows,numCols)); 
  }
  
  /** 
    Find the Norm of two matrices 
    */
  public static double matrixNorm(double[][] A, int Ai, int Aj, double[][] B, int Bi, int Bj, int numRows, int numCols){
    
    double sum = 0.0;

    for (int i = 0; i < numRows; i++)
      for(int j = 0; j < numCols; j++)
	sum += Math.pow((A[i + Ai][j + Aj] - B[i + Bi][j + Bj]), 2.0);
    return (Math.sqrt(sum)/(double)(numRows * numCols));
  }

  /*
  public static double matrixNorm(double[][] A, int Ai, int Aj, 
				  double[][] B, int Bi, int Bj, int numRows, int numCols){
  
    double sum1 = 0.0;
    double sum2 = 0.0;
    double sum3 = 0.0;
    for (int i=0; i<numRows; i++)
      for (int j=0; j<numCols; j++)
	{
	  double Aij = A[i+Ai][j+Aj];
	  double Bij = B[i+Bi][j+Bj];
	  sum1 += (Aij-Bij)*(Aij-Bij);
	  sum2 += Aij*Aij;
	  sum3 += Bij*Bij;
	}

    return sum1/(sum2+sum3);
  }
  */


  /**
    Save a dense matrix to file.
   */
  public static void save(final double[][] matrix, final String fileName) throws IOException {
    
    PrintWriter pw = new PrintWriter(new FileWriter(fileName));
    int tempHeight = matrix.length;
    int tempWidth = matrix[0].length;
    
    pw.println(tempHeight + " " + tempWidth);

    for (int i = 0; i < tempHeight; i++) {
      for (int j = 0; j < tempWidth; j++)
	pw.print(matrix[i][j] + " ");
      pw.println();
    }

    pw.close();
  }


  /**
    Save a dense submatrix to file.
    */
  public static void save(final double[][] matrix, final int rowOffset, final int colOffset,
				final int M, final int N, final String fileName) throws IOException {
    				  
    PrintWriter pw = new PrintWriter(new FileWriter(fileName));
    
    pw.println(M + " " + N);

    for (int i = rowOffset; i < rowOffset + M; i++) {
      for (int j = colOffset; j < colOffset + N; j++)
	pw.print(matrix[i][j] + " ");
      pw.println();
    }

    pw.close();
  }

  /**
    Save a dense vector to file.
   */
  public static void save(final double[] vector, int incv, final String fileName) throws IOException {
    
    PrintWriter pw = new PrintWriter(new FileWriter(fileName));
    int tempHeight = vector.length;

    // write the length
    pw.println(tempHeight);

    for (int i = 0; i < tempHeight; i++) {
      pw.println(vector[i]);
    }

    pw.close();
  }


  /** 
    Print a dense submatrix.
    */
  public static void print(final double[][] matrix) {
    System.out.println(matrix.length + " " + matrix[0].length);
    for (int i = 0; i < matrix.length; i++) {
      for (int j = 0; j < matrix[0].length; j++) 
	System.out.print(matrix[i][j]+ "  ");
      System.out.println();
    }
  }

  /** 
    Print a vector
    */
  public static void printVector(final double[] vector) {
    System.out.println(vector.length);
    for (int i = 0; i < vector.length; i++) 
      System.out.println(vector[i]);
  }
}
