//package jnt.io;

import java.io.*;

/*
 * This class provides an easy to read/write basic data types
 * from character streams.  (You'd think this would be provided
 * as part of the JDK java.lang.io package.)
 *
 *
 *
 *
 *
 * Gary Cornell and Cay S. Horstmann, Core Java (Book/CD-ROM)
 * Published By SunSoft Press/Prentice-Hall
 * Copyright (C) 1996 Sun Microsystems Inc.
 * All Rights Reserved. ISBN 0-13-565755-5
 *
 * Permission to use, copy, modify, and distribute this 
 * software and its documentation for NON-COMMERCIAL purposes
 * and without fee is hereby granted provided that this 
 * copyright notice appears in all copies. 
 * 
 * THE AUTHORS AND PUBLISHER MAKE NO REPRESENTATIONS OR 
 * WARRANTIES ABOUT THE SUITABILITY OF THE SOFTWARE, EITHER 
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A 
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. THE AUTHORS
 * AND PUBLISHER SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED 
 * BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING 
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */
 
 /**
	An easy interface to read numbers and strings from 
	standard input.

    <p>
	Example use:
	<pre>

	NumericReader R = new NumericReader( new FileReader("foo.dat"));

	double x = R.readDouble();
	int    i = R.readInt();
	String s = R.readWord();
	String l = R.readLine();

	public variable:  R.Reader_ can be used in functions

	</pre>
or	
	<pre>
	NumericReader R = new NumericReader(new InputStreamReader(System.in));
	</pre>

*/
public class NumericReader 
{  

	public Reader Reader_;
	private static char r[] = new char[80];		// used in readWord();

		
	public NumericReader(Reader in)
	{
		Reader_ = in;
	}
	  
   /**
    * read a line from a charater input stream. The string is 
    * terminated by a newline
    * @return the input line (without the newline)
    */
   

   public String readLine()
   {  int ch;
      String r = "";
      boolean done = false;
      while (!done)
      {  try
         {  ch = Reader_.read();
            if (ch < 0 || (char)ch == '\n')
               done = true;
            else
               r = r + (char) ch;
         }
         catch(java.io.IOException e)
         {  done = true;
         }
      }
      return r;
   }


   /**
    * read a word from the console. The word is 
    * the first set of contiguous non-whitespace characters
    * @return the 'word' entered
    */
    
   public String readWord() throws java.io.IOException
   {  
   	  int ch;
      boolean done = false;
	  int len = 0; 

	  try
	  {
	  	// eat-up white space (POZO: modification)
		//
	  	ch  = Reader_.read();
	 	while (java.lang.Character.isWhitespace((char)ch))
			ch = Reader_.read();

	   // try to process first character
	   //
       if (ch < 0 || java.lang.Character.isWhitespace((char)ch))
          done = true;
       else
	   {
	   	  r[len] = (char) ch;		// was r = r + (char) ch;
		  len++;
		}
			   
	   while (!done)
        {  
		   ch = Reader_.read();  
           if (ch < 0 || java.lang.Character.isWhitespace((char)ch))
               done = true;
            else
			{
              r[len] = (char) ch;  // was  r = r + (char) ch;
			  len++;
			}
         }
	}
    catch(java.io.IOException e) {  done = true; }
    
	  
    // System.out.println("readWord() returned: [" + r + "]");
    return new String(r, 0, len);
   }

   /**
    * read an integer from the console. The input is 
    * terminated by a newline
    * @param prompt the prompt string to display
    * @return the input value as an int
    * @exception NumberFormatException if bad input
    */
    
   public int readInt() throws java.io.IOException
   {
   		int value = 0;
         try
         {  value = Integer.valueOf
               (readWord().trim()).intValue();
         } 
		 catch(NumberFormatException e)
         {  
		 	System.out.println
               ("expected integer.");
         }
		 return value;
   }

   /**
    * read a floating point number from the console. 
    * The input is terminated by a newline
    * @param prompt the prompt string to display
    * @return the input value as a double
    * @exception NumberFormatException if bad input
    */
    
   public double readDouble() throws java.io.IOException
   {  
   		double value = 0.0;
         try
         {  
		 	value = Double.valueOf
               (readWord().trim()).doubleValue();
         } 
		 catch(NumberFormatException e)
         {  
		   System.out.println( 
			"NumericReader.readDouble(): expecting a floating point number.");
         }
		 return value;
   }
   
  
}


